
{_____________________________________________________________________
                           F4n1024.DSP

        Optimized complex 1024-Point Radix-4 DIF FFT Program for the
        Analog Devices ADSP-210X Digital Signal Processor Family

        This routine uses a slightly modified radix_4 algorithm to 
        effectively descramble results as they are computed.  The
        result is in_order output.
        
        Complex data stored X0,Y0, X1, Y1,... (real, imag, real, imag)

        The following terminology will be used to describe
        the  "legs" of the radix-4 butterfly:
        xa = 1st real input leg        x'a = 1st real output leg
        xb = 2nd real input leg        x'b = 2nd real output leg
        xc = 3rd real input leg        x'c = 3rd real output leg 
        xd = 4th real input leg        x'd = 4th real output leg
        ya = 1st imag input leg        y'a = 1st imag output leg
        yb = 2nd imag input leg        y'b = 2nd imag output leg
        yc = 3rd imag input leg        y'c = 3rd imag output leg
        yd = 4th imag input leg        y'd = 4th imag output leg

        The index registers of the 210X will be used as pointers
        to the following:
        I0 --> xa,xc
        I1 --> xb,xd
        I2 --> ya,yc
        I3 --> yb,yd                
               w0 (= Ca = Sa = 0)
        I5 --> w1 (1st Cb, - pi/4 for Sc)
        I6 --> w2 (2nd Cc, - pi/4 for Sc)
        I7 --> w3 (3rd Cd, - pi/4 for Sd)

        input : inplace[2*N] normal order, interleaved real,imag.
        output: inplace[2*N] normal order, interleaved real,imag.



        Benchmark:
          setup  =     9
          stage 1=     7700 = 20+256(30)
          stage 2=     7758 = 18+4(15+64(30))
          stage 3=     7938 = 18+16(15+16(30))
          stage 4=     8658 = 18+64(15+4(30))
          stage 5=     5140 = 20+256(20)

          Total       37203 cycles * 80ns = 2.97624ms

_____________________________________________________________________}

.MODULE/BOOT=0   FFT_SUB;
.const           N = 1024; 
.const           NX2 = 2048; 
.const           NOV2 = 512; 
.const           NOV4 = 256; 
.const           N3OV8 = 384; 
.VAR/DM/CIRC     COS_TABLE[1024];

.VAR/DM          M3_SPACE;      {memory space used to store M3 values}
                                {when M3  is loaded with its alternate 
                                 value}

.VAR/DM          BFY_COUNT;
.INIT            COS_TABLE:<cos1024.dat>;        {N cosine values}
.ENTRY           FFT;
.EXTERNAL        INPLACE;

FFT: M4=-Nov4;          {-N/4 = -90 degrees for sine}
     L0=0;
     L1=0;
     L2=0;
     L3=0;
     L5=%COS_TABLE;
     L6=%COS_TABLE;
     L7=%COS_TABLE;
     SE=0;

{_____________________________Stage 1_______________________________}



STAGE1: I0=^INPLACE;                    {in       ->Xa,Xc}
        I1=^INPLACE+Nov2;               {in+N/2   ->Xb,Xd}
        I2=^INPLACE+1;                  {in+1     ->Ya,Yc}
        I3=^INPLACE+Nov2+1;             {in+N/2+1 ->Yb,Yd}
        I5=^COS_TABLE;
        I6=^COS_TABLE;
        I7=^COS_TABLE;
        M0=N;               {N,    skip forward to dual node}
        M1=-N;              {-N,   skip back to primary node}
        M2=-N+2;            {-N+2, skip to next butterfly}
        
        M3=-2;              {because we have modified the middle branches}
                            {of bfly, pointers for I0 have to be treated a}
                            {little more carefully.  This requires a more}
                            {complex pointer manipulation.  M3 is used in }
                            {doing this}
 
        M5=Nov4+1;          {N/4 + groups/stage*1, Cb Sb offset}
        M6=Nov4+2;          {N/4 + groups/stage*2, Cc Sc offset}
        M7=Nov4+3;          {N/4 + groups/stage*3, Cd Sc offset}
        AX0=DM(I0,M0);                        {get first Xa}
        AY0=DM(I0,M1);                        {get first Xc}
        AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya}
        SR=LSHIFT AR(LO), AY1=DM(I2,M1);      {SR1=Xa-Xc,get first Yc}
        CNTR=Nov4;                            {Bfly/group, stage one}

  {This radix_4 bfly algorithm is a modified one.  It has it's middle 2 }
  {Branches reversed.  This alteration, done to every stage, results in }
  {a bit_reversed output instead of a digit reversed output             }



        DO STG1BFY  UNTIL CE;
                AR=AX0+AY0, AX0=DM(I1,M0);
                           {AR=xa+xc, AX0=xb }
                MR0=AR, AR=AX1+AY1;
                           {MR0=xa+xc, AR=ya+yc}
                MR1=AR, AR=AX1-AY1;
                           {MR1=ya+yc, AR=ya-yc}
                SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1);
                           {SR1=ya-yc, AY0=xd}
                AF=AX0+AY0, AX1=DM(I3,M0);
                           {AF=xb+xd, AX1=yb}
                AR=MR0+AF, AY1=DM(I3,M1);
                           {AR=xa+xb+xc+xd, AY1=yd}
                DM(I0,M0)=AR, AR=MR0-AF;
                           {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd}
                AF=AX1+AY1, MX0=AR;
                           {AF=yb+yd, MX0=xa+xc-xb-xd}
                AR=MR1+AF, MY0=DM(I6,M4);
                           {AR=ya+yc+yb+yd, MY0=(Cc)}
                DM(I2,M0)=AR, AR=MR1-AF;
                           {output y'a, AR=ya+yc-yb-yd}
                MR=MX0*MY0(SS), MY1=DM(I6,M6);
                           {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)}
                MR=MR+AR*MY1(RND),SI=DM(I0,M2);    
                           {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                           {si here is a dummy to perform a modify(I0,M2)}
                DM(I1,M0)=MR1, MR=AR*MY0(SS);
                           {output x'c to  positionx'b}
                           {MR=(ya+yc-yb-yd)(Cc)}
                MR=MR-MX0*MY1(RND), MY0=DM(I5,M4);
                           {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)}
                DM(I3,M0)=MR1, AR=AX0-AY0;
                           {output y'c=to position y'b}
                           {AR=xb-xd}
                AY0=AR, AF=AX1-AY1;
                           {AY0=xb-xd, AF=yb-yd}
                AR=SR0-AF, MY1=DM(I5,M5);
                           {AR=xa-xc-(yb-yd), MY1=(Sb)}
                MX0=AR, AR=SR0+AF;
                           {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd}
                SR0=AR, AR=SR1+AY0;
                           {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd}
                MX1=AR, AR=SR1-AY0;
                           {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)}
                MR=SR0*MY0(SS),AX0=DM(I0,M0);
                           {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly}
                MR=MR+AR*MY1(RND),AY0=DM(I0,M3);
                           {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                           {AY0= xc of next bfly}
                DM(I0,M2)=MR1, MR=AR*MY0(SS);
                           {output x'b=to position x'c}
                           {MR=ya-yc-xb+xd)(Cb)}
                MR=MR-SR0*MY1(RND), MY0=DM(I7,M4);
                           {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                           {MY0=(Cd)}
                DM(I2,M2)=MR1, MR=MX0*MY0(SS);
                           {ouput y'b= to position y'c}
                           {MR=(xa-yb-xc+yd)(Cd)}
                MY1=DM(I7,M7), AR=AX0-AY0;
                            {MY1=(Sd), AR=xa-xc }  
                MR=MR+MX1*MY1(RND), AX1=DM(I2,M0);
                           {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {AX1=ya of next bfly}
                DM(I1,M2)=MR1, MR=MX1*MY0(SS);
                           {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {MR=(ya+yb-yc-yd)(Cd)}
                MR=MR-MX0*MY1(RND), AY1=DM(I2,M1);
                           {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd}
                           {yc of next bfly}
STG1BFY:        DM(I3,M2)=MR1, SR=LSHIFT AR(LO);
                           {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd}
                           {SR0=ya-yc of next bfly}



{_____________________________Stage 2_______________________________}



STAGE2:     I0=^INPLACE;       {in       -> Xa,Xc}
            I1=^INPLACE+128;   {in+N/8   -> Xb,Xd}
            I2=^INPLACE+1;     {in+1     -> Ya,Yc}
            I3=^INPLACE+129;   {in+N/8+1 -> Yb,Yd}
            M0=Nov4;           {N/4,    skip forward to dual node}
            M1=-Nov4;          {-N/4,   skip back to primary node}
            M2=-Nov4+2;        {-N/4+2, skip to next butterfly}
    
            M3=384;                     {N*3/8,  skip to next group}
            DM(M3_SPACE)=M3;            {M3_space is  temporary storage}
                                        {space needed because M3 isused}
                                        {in 2 contexts and will alternate}
                                        {in value}
           
           M5=Nov4+4;         {N/4 +groups/stage*1, Cb Sb offset}
           M6=Nov4+8;         {N/4 +groups/stage*2, Cc Sc offset}
           M7=Nov4+12;        {N/4 +groups/stage*3, Cd Sd offset}
           SI=64;             {Bfy/group, save counter for inner loop} 
           DM(BFY_COUNT)=SI;  {SI is used as a temporary storage dummy}

           CNTR=4;            {groups/stage}
           CALL MID_STG;      {do stage 2}

{_____________________________Stage 3_______________________________}

STAGE3:     I0=^INPLACE;        {in         -> Xa,Xc}
            I1=^INPLACE+32;     {in+N/32    -> Xb,Xd}
            I2=^INPLACE+1;      {in+1       -> Ya,Yc}
            I3=^INPLACE+33;     {in+N/32+1  -> Yb,Yd}
            M0=64;              {N/16,    skip forward to dual node}
            M1=-64;             {-N/16,   skip back to primary node}
            M2=-62;             {-N/16+2, skip to next butterfly}
            M3=96;                      {N*3/32,  skip to next group}
   
            DM(M3_SPACE)=M3;            {M3_space is  temporary storage}
                                        {space needed because M3 isused}
                                        {in 2 contexts and will alternate}
                                        {in value}
           
           M5=Nov4+16;         {N/4 +groups/stage*1, Cb Sb offset}
           M6=Nov4+32;         {N/4 +groups/stage*2, Cc Sc offset}
           M7=Nov4+48;         {N/4 +groups/stage*3, Cd Sd offset}
           SI=16;              {Bfy/group, save counter for inner loop} 
           DM(BFY_COUNT)=SI;   {SI is a dummy temporary register here }
 
           CNTR=16;            {groups/stage}
           CALL MID_STG;       {do stage 3}

{_____________________________Stage 4_______________________________}
stage4:     I0=^INPLACE;       {in          -> Xa,Xc}
            I1=^INPLACE+8;     {in+N/128    -> Xb,Xd}
            I2=^INPLACE+1;     {in+1        -> Ya,Yc}
            I3=^INPLACE+9;     {in+N/128+1  -> Yb,Yd}
            M0=16;             {N/64,    skip forward to dual node}
            M1=-16;            {-N/64,   skip back to primary node}
            M2=-14;            {-N/64+2, skip to next butterfly}

            M3=24;                      {N*3/128, skip to next group}
            DM(M3_SPACE)=M3;            {M3_space is  temporary storage}
                                        {space needed because M3 isused}
                                        {in 2 contexts and will alternate}
                                        {in value}
            

           M5=Nov4+64;        {N/4 +groups/stage*1, Cb Sb offset}
           M6=Nov4+128;       {N/4 +groups/stage*2, Cc Sc offset}
           M7=Nov4+192;       {N/4 +groups/stage*3, Cd Sd offset}
           SI=4;              {Bfly/group, save counter inner loop}
           DM(BFY_COUNT)=SI;  {SI is a dummy used for storage }

           CNTR=64;           {groups/stage}
           CALL MID_STG;      {do stage 4}

{_________________Last Stage__No Multiplies__________________________}


                            
laststage:  I4=^INPLACE;       {in         ->Xa,Xc}
            I5=^INPLACE+2;     {in+N/512   ->Xb,Xd}
            I6=^INPLACE+1;     {in+1       ->Ya,Yc}
            I7=^INPLACE+3;     {in+N/512+1 ->Yb,Yd}
            M4=4;              {N/256,  skip forward to dual node}
          
            M0=h#0010;          {this magic modify value is used to  }
                                {perform a bit reverse as the final results}
                                { are written out.  Because of interleave}
                                {this is not as simple as explained in }
                                {1410 Handbook.  Derivation of these values}
                                {is in the Apps. note of this code}

           I0=h#0000;          {these  base address values are derived }
           I2=h#2000;          {for an output at address 0000.}
           I1=h#0008;          
           I3=h#2008;
           
           L4=0;               {this last stage has no twiddles and}
           L5=0;               {because it is where the DAG#1 bit_}
           L6=0;               {reverses the output, the I's M's & L's}
           L7=0;               {have been reassigned and re_initialized}

           AX0=DM(I4,M4);      {first Xa} 
           AY0=DM(I4,M4);      {first Xc}
           CNTR=Nov4;          {groups/stage}
           ENA BIT_REV;        {all data accesses using I0..I3 are}
                               {bit-reversed  }


  {radix_4 bfly algorithm is a modified one.  It has it's middle 2 }
  {Branches reversed.  This alteration, done to every stage, results in }
  {a bit_reversed output instead of a digit reversed output             }



         DO LASTSTGBFY UNTIL CE;
                AR=AX0-AY0, AX1=DM(I6,M4);       {AR=xa-xc, AX1=ya}
                SR=LSHIFT AR(LO), AY1=DM(I6,M4); {SR0=xa-xc, AY1=yc}
                AR=AX0+AY0, AX0=DM(I5,M4);       {AR=xa+xc, AX0=xb}
                MR0=AR, AR=AX1+AY1;              {MR0=xa+xc, AR=ya+yc}
                MR1=AR, AR=AX1-AY1;              {MR1=ya+yc, AR=ya-yc}
                SR=SR OR LSHIFT AR (HI), AY0=DM(I5,M4);
                                                 {SR1=ya-yc, AY0 xd}
                AF=AX0+AY0, AX1=DM(I7,M4);       {AF=xb+xd, AX1=yb}
                AR=MR0+AF, AY1=DM(I7,M4);        {AR=xa+xc+xb+xd, AY1=yd}
                DM(I0,M0)=AR, AR=MR0-AF;         {output x'a=xa+xc+xb+xd}
                                                 {AR=xa+xc-(xb+xd)}
                DM(I1,M0)=AR, AF=AX1+AY1;        {output x'c to position x'b}
                                                 {AF=yb+yd}
                AR=MR1+AF;                       {AR=ya+yb+yc+yd}
                DM(I2,M0)=AR, AR=MR1-AF;         {output y'a=ya+yc+yb+yd}
                                                 {AR=ya+yc-(yb+yd)}
                DM(I3,M0)=AR, AR=AX0-AY0;        {output y'c to position y'b}
                                                 {AR=xb-xd}
                AX0=DM(I4,M4);                   {AX0=xa of next group}
                AF=AX1-AY1, AY1=AR;              {AF=yb-yd, AY1=xb-xd}
                AR=SR0+AF, AY0=DM(I4,M4);        {AR=xa-xc+yb-yd}
                                                 {AY0=xc of next group}
                DM(I0,M0)=AR, AR=SR0-AF;         {output x'b to position x'c}
                                                 {AR=xa-xc-(yb-yd)}
                DM(I1,M0)=AR, AR=SR1-AY1;        {output x'd=xa-xc-(yb-yd)}
                                                 {AR=ya-yc+(xb-xd)}
                DM(I2,M0)=AR, AR=SR1+AY1;        {output y'b to position y'c}
                                                 {AR=ya-yc-(xb-xd)}
laststgbfy:     DM(I3,M0)=AR;                    {output y'd=ya-yc-(xb-xd)}

                DIS  BIT_REV;                    {shut-off bit reverse}
                                                 {mode, not necessary}
                RTS;                             {end and exit from FFT
                                                  subroutine} 



{_____________________Subroutine for middle stages___________________}


mid_stg:  DO MIDGRP UNTIL CE;
              I5=^COS_TABLE;
              I6=^COS_TABLE;
              I7=^COS_TABLE;
              AX0=DM(I0,M0);                        {get first Xa}
              AY0=DM(I0,M1);                        {get first Xc}
              AR=AX0-AY0, AX1=DM(I2,M0);            {Xa-Xc,get first Ya}
              SR=LSHIFT AR (LO), AY1=DM(I2,M1);     {SR1=Xa-Xc,get first Yc}
              CNTR=DM(BFY_COUNT);                   {butterflies/group}


              M3=-2;               {M3 is loaded with the special value}
                                   {required for pointer manipulation }
                                   {as explained previously}



  {This radix_4 bfly algorithm is a modified one.  It has it's middle 2  }
  {Branches reversed.  This alteration, done to every stage, resdults in }
  {a bit reversed output instead of a digit reversed output              }

           DO MIDBFY UNTIL CE;
                AR=AX0+AY0, AX0=DM(I1,M0);
                           {AR=xa+xc, AX0=xb }
                MR0=AR, AR=AX1+AY1;
                           {MR0=xa+xc, AR=ya+yc}
                MR1=AR, AR=AX1-AY1;
                           {MR1=ya+yc, AR=ya-yc}
                SR=SR OR LSHIFT AR (HI), AY0=DM(I1,M1);
                           {SR1=ya-yc, AY0=xd}
                AF=AX0+AY0, AX1=DM(I3,M0);
                           {AF=xb+xd, AX1=yb}
                AR=MR0+AF, AY1=DM(I3,M1);
                           {AR=xa+xb+xc+xd, AY1=yd}
                DM(I0,M0)=AR, AR=MR0-AF;
                           {output x'a=(xa+xb+xc+xd), AR=xa+xc-xb-xd}
                AF=AX1+AY1, MX0=AR;
                           {AF=yb+yd, MX0=xa+xc-xb-xd}
                AR=MR1+AF, MY0=DM(I6,M4);
                            {AR=ya+yc+yb+yd, MY0=(Cc)}
                DM(I2,M0)=AR, AR=MR1-AF;
                           {output y'a, AR=ya+yc-yb-yd}
                MR=MX0*MY0(SS), MY1=DM(I6,M6);
                           {MR=(xa+xc-xb-xd)(Cc), MY1=(Sc)}
                MR=MR+AR*MY1(RND), SI=DM(I0,M2);
                           {MR=(xa-xb+xc-xd)(Cc)+(ya-yb+yc-yd)(Sc)}
                           {SI is a dummy here to cause a modify(I0,M2)}   
                DM(I1,M0)=MR1, MR=AR*MY0(SS);
                           {output x'c to position x'b}
                           {MR=(ya+yc-yb-yd)(Cc)}
                MR=MR-MX0*MY1(RND), MY0=DM(I5,M4);
                           {MR=(ya+yc-yb-yd)(Cc)-(xa+xc-xb-xd)(Sc), MY0=(Cb)}
                DM(I3,M0)=MR1, AR=AX0-AY0;
                           {output y'c to position y'b}
                           {AR=xb-xd}
                AY0=AR, AF=AX1-AY1;
                           {AY0=xb-xd, AF=yb-yd}
                AR=SR0-AF, MY1=DM(I5,M5);
                           {AR=xa-xc-(yb-yd), MY1=(Sb)}
                MX0=AR, AR=SR0+AF;
                           {MX0=xa-xc-yb+yd, AR=xa-xc+yb-yd}
                SR0=AR, AR=SR1+AY0;
                           {SR0=xa-xc+yb-yd, AR=ya-yc+xb-xd}
                MX1=AR, AR=SR1-AY0;
                           {MX1=ya-yc+xb-xd, AR=ya-yc-(xb-xd)}
                MR=SR0*MY0(SS),  AX0=DM(I0,M0);
                          {MR=(xa-xc+yb-yd)(Cb), AX0= xa of next bfly}
                MR=MR+AR*MY1(RND),  AY0=DM(I0,M3);
                           {MR=(xa-xc+yb-yd)(Cb)+ (ya-yc-xb+xd)(Sb)}
                           {AY0= xc of next bfly}
                DM(I0,M2)=MR1, MR=AR*MY0(SS);
                           {ouput x'b to position x'c}
                           {MR=ya-yc-xb+xd)(Cb)}
                MR=MR-SR0*MY1(RND), MY0=DM(I7,M4);
                           {MR=(ya-yc-xb+xd)(Cb)-(xa-xc+yb-yd)(Sb)}
                           {MY0=(Cd)}
                DM(I2,M2)=MR1, MR=MX0*MY0(SS);
                           {ouput y'b to position y'c}
                           {MR=(xa-yb-xc+yd)(Cd)}
                MY1=DM(I7,M7), AR=AX0-AY0;
                            {MY1=(Sd), AR=xa-xc }  
                MR=MR+MX1*MY1(RND), AX1=DM(I2,M0);
                           {MR=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {AX1=ya of next bfly}
                DM(I1,M2)=MR1, MR=MX1*MY0(SS);
                           {output x'd=(xa-yb-xc+yd)(Cd)+(ya+xb-yc-xd)(sd)}
                           {MR=(ya+yb-yc-yd)(Cd)}
                MR=MR-MX0*MY1(RND), AY1=DM(I2,M1);
                           {MR=(ya+yb-yc-yd)(Cd)-(xa-xc-yb+yd)Sd}
                           {yc of next bfly}
midbfy:         DM(I3,M2)=MR1, SR=LSHIFT AR(LO);
                           {output y'd=(ya+xb-yc-xd)Cd-(xa-xc-yb+yd)Sd}
                           {SR0=ya-yc of next bfly}
        
          M3=DM(M3_SPACE);    {modifier M3 is loaded with 'skip to}
                              {next group'_count and is used in the}
                              {next four instructions}

          MODIFY (I0,M3);
          MODIFY (I1,M3);             { point to next group }
          MODIFY (I2,M3);             { of butterflies }
midgrp:   MODIFY (I3,M3);
RTS;                                  {return to middle stage calling code}
{_____________________________End code_______________________________}
.ENDMOD;

