{****************************************************************************
ADSP-2111 - AD1849(SOUNDPORT)  INTERFACE PROGRAM
Oli Josefsson & Kapriel Karagozyan, Analog Devices     Sept, 1991

This program is written to run on an ADSP-2111. However, it can be easily 
changed to work with an ADSP-2101 by modifying the interrupt vector table. 
For the hardware connections between the ADSP-2111 and AD1849 please refer
to the diagrams shown on the AD1849 data sheet.

The AD1849 will be set up as follows:
(NC indicates a no care state, all control reg are 1 bit unless indicated 
 by [#bits])

Control time slot control bits:
DCB=0 followed by 1
AC=1    autocalibrate
DFR=5   Data conversion frequency,44.1kH [3]
ST=1    stereo mode
DF=0    dataformat is 16 bit twos complement [2]
MCK=2   16.9344 MHz is the master clock [2]
FSEL=0  frame size, 64 bits [2]
MS=1    master mode (i.e. receive external serial clock)
TXDIS=0 enable serial output
ENL=0   disable loopback testing
ADL=NC  loopback mode analog/digital (disabled)
PIO=NC  paralell I/O bits not used [2]
REVID=NC

Data Timeslot Control Bits:
OM0=1    enable line 0 output
OM1=1    enable line 1 output
LO=0    no attenuation of left channel output [6]
SM=0    mono output is muted
RO=0    no attenuation of right channel output [6]
PIO=NC  paralell I/O bits not used [2]
OVR=NC  overrange INPUT
IS=0    line-level stereo input selected
LG=0    no gain for left channel [4]
MA=15   no monitor mix (i.e. ADC output is not mixed with DAC input
RG=0    no gain for right channel [4]

*****************************************************************************

This program makes use of the multi-channel mode that is available on the
SPORT0 and it also uses autobuffering to reduce interrupt service overhead.
For a description of the multi-channel and autobuffering features, please
refer to the ADSP-2111 or ADSP-2101 architecture user's manuals.

In its current condition, this program can be used "as is" to perform straight
talkthrough (at 44.1 KHz sample rate) on both left and right channels of the 
AD1849. The incoming audio data from the 16bit ADCs is placed in a short buffer 
and immediately sent out to the 16bit DACs.

The initial setup and handshaking with the AD1849 starts with the "START"
routine. A state machine to perform the handshaking is executed in software.
Once the AD1849 is configured properly, the processor enters the WAIT_DATA
loop and waits for serial port interrupt requests.

This program is booted into the ADSP-2111 on power-up.

*****************************************************************************

Modified by Charles Steinkuehler to correct interrupt vector table and 
SPORT1/Flag_Out Differences


*****************************************************************************}

.MODULE/ABS=0/BOOT=0  AD1849;
.VAR/CIRC  CTRLIN[4];             {circular buffers for data input and}
.VAR/CIRC  CTRLOUT[4];            {output for data mode and control mode}
.VAR/CIRC  DATAIN[4];
.VAR/CIRC  DATAOUT[8];
.VAR   	   FIRST_FLG;
.VAR       DCB_FLG;
.VAR       DMODE_FLG;

                 JUMP START;nop;nop;nop;         {restart interrupt}
                 RTI;nop;nop;nop;                {IRQ2 int, not used}
                 JUMP SETUPCONTROL;nop;nop;nop;  {SPORT0 transmit int}
                 JUMP NEWDATA;nop;nop;nop;       {SPORT0 receive int}
                 RTI;nop;nop;nop;                {SPORT1 tx or IRQ1 int, not used}
                 RTI;nop;nop;nop;                {SPORT1 rx or IRQ0 int, not used}
                 RTI;nop;nop;nop;                {timer int,not used}

START:           L0=0X001F;	    {Enable Flag_Out}
                 DM(0X3FFF)=L0;
                 RESET FLAG_OUT;    {set the AD1849 D/C pin low}
		 L0=%CTRLIN;
                 M0=1;
                 I0=^CTRLIN;
      		 L1=%CTRLOUT;
     		 M1=1;
      		 I1=^CTRLOUT;
                 AX0=1;
                 DM(FIRST_FLG)=AX0;
                 DM(DCB_FLG)=AX0;
                 AX0=0;
                 DM(DMODE_FLG)=AX0;
                 AY0=B#0010000100101100; 

{Initialize contol mode output buffer}
                 
                 DM(I1,M1)= B#0010000100101100; {DCB=0,AC=1,DFR=05,ST=1,DF=00}
                 DM(I1,M1)= B#0010001000000000; {MCK=02,FSEL=0,MS=0,TXDIS=0}
                 				{ENL=0,ADL=0}
                 DM(I1,M1)= B#0000000000000000; {PIO = 00}
                 DM(I1,M1)= B#0000000000000000; {REVID=NC}
                 L2 = 0;                  {linear addressing for register }
                 I2 = 0x3fef;             {pt. to last DM cntrl reg}
{3FEF}           DM(I2,M1) = 0x0000;      {sport1 not used}
{3FF0}           DM(I2,M1) = 0x0000;
{3FF1}           DM(I2,M1) = 0x0000;
{3FF2}           DM(I2,M1) = 0x0000;
{3FF3}           DM(I2,M1) = 0X0283; {autobuf. rx:i0, m0 tx:i1,m1}
{3FF4}           DM(I2,M1) = 383;    {sport rfsdiv, sets up FRAME sync freq}
{3FF5}           DM(I2,M1) = 849;    {sport0 sclkdiv, SCLK will be less than}
                                     {8KHz from control mode}
{3FF6}           DM(I2,M1) = B#1100010100011111;
                                      {sport0 control register:
                                       multi-channel mode, 24 channels
                                       internal sclk & rfs
                                       normal framing mode
                                       frame sync not inverted
                                       16 bit word length}
{3FF7}           DM(I2,M1) = 0x000F;  {first 4 xmit multichannels used}
{3FF8}           DM(I2,M1) = 0x0000;
{3FF9}           DM(I2,M1) = 0x000F;  {first 4 rx multichannels used}
{3FFA}           DM(I2,M1) = 0x0000;
{3FFB}           DM(I2,M1) = 0x0000;  {TSCALE register}
{3FFC}           DM(I2,M1) = 0x0000;  {TCOUNT register}
{3FFD}           DM(I2,M1) = 0x0000;  {TPERIOD register,initializing value
                                       for TCOUNT after every interrupt}
{3FFE}           DM(I2,M1) = 0x0000;  {external data memory waits=0}


                 ICNTL=0x00 ;
                 IMASK=B#00010000;    {only SPORT transmit intrpt enabled
                                       initially while in control mode}

                 AX0=DM(I1,M1);       {send first 16bits of ctrl word}
                 TX0=AX0;
{3FFF}           DM(I2,M1) = 0x1018;  {system control reg: sport0 enabled}

{Wait for an interrupt indicating that transmit register is ready for new data
 and that the 2111 has received a 16bit word}

WAIT1:           AX1=DM(DMODE_FLG);
                 AR=PASS AX1;
                 IF GT JUMP GO_DMODE;
                 JUMP WAIT1;

SETUPCONTROL:    AX0=DM(FIRST_FLG);
                 AF=PASS AX0;
                 IF NE JUMP DECR_FIRST;
                 AX0=DM(DCB_FLG);
		 AR=PASS AX0;
		 IF EQ JUMP DCBFLG_SET;
                 AX0=DM(CTRLIN);	{DCB_FLG has not been set yet}
                 AR=AX0 XOR AY0;        {check all incoming bits including DCB bit}
                 IF EQ JUMP SET_DCB;	{set flag if DCB was 0}
                 RTI;
DCBFLG_SET:	 AX0=DM(CTRLIN);	{DCB_FLG was set}
		 AR=AX0 AND AY0; 	{only check for DCB bit}
		 IF NE JUMP SETDMODE;	{if DBC=1 ready for datamode}
                 RTI;

SET_DCB:         AX0=0;
                 DM(DCB_FLG)=AX0;
                 AX0=B#0010010100101100; {DCB was 0, prepare to send DCB=1}
                 DM(CTRLOUT)=AX0;
		 AY0=B#0000010000000000; 
                 RTI;

DECR_FIRST:	 AX0=0;
		 DM(FIRST_FLG)=AX0;
                 RTI;

SETDMODE:        IMASK=0;
                 AX0=0X0018;              {disable sport0}
                 DM(0X3FFF)=AX0;

{ At this point we could boot page#1 and do the following data mode setup after
  the reboot occurs.  This would free up some PM RAM for other uses}
                 
                 I1 = ^DATAOUT;
                 L1=0;
                 DM(I1,M1) = 0x0000;             {reset output & input data buffers}
                 DM(I1,M1) = 0x0000;             {initialize embedded control bits}
                 DM(I1,M1) = B#1100000001000000; {OM1=1,OM1=1,LO=0,SM=1,RO=0}
                 DM(I1,M1) = B#0000000011110000; {PIO=00,OVR=1,IS=0,LG=0,MA=15,RG=0}
                 DM(I1,M1) = 0x0000;             {reset output & input data buffers}
                 DM(I1,M1) = 0x0000;             {initialize embedded control bits}
                 DM(I1,M1) = B#1100000001000000; {OM1=1,OM1=1,LO=0,SM=1,RO=0}
                 DM(I1,M1) = B#0000000011110000; {PIO=00,OVR=1,IS=0,LG=0,MA=15,RG=0}
                 AX0=0X861F;
                 DM(0X3FF6)=AX0;        {sport0 control:
               				multi-channel mode
			                external sclk & rfs
 			                32 word frames, 16bit words}
                 SET FLAG_OUT;               {set D/C high}
                 AX0=1;
                 DM(DMODE_FLG)=AX0;
                 RTI;

GO_DMODE:        L0=%DATAIN;
                 I0=^DATAIN;
                 L1=%DATAOUT;
                 I1=^DATAOUT;
                 M2=3;
                 L3=L1;
		 AX0=0XFFFF;
		 DM(0X3FF7)=AX0;        {enable all multi-channel words}
		 DM(0X3FF8)=AX0;
		 DM(0X3FF9)=AX0;
		 DM(0X3FFA)=AX0;
                 AX0=DM(I1,M1);         {send first 16bits of data}
                 TX0=AX0;
                 AX0=0X1018;
                 DM(0X3FFF)=AX0;        {turn on sport0}
                 IFC=B#000000111111;	{clear all pending interrupts}
                 IMASK=B#00001000;      {sport0 rx interrupt on}

WAIT_DATA:       JUMP WAIT_DATA;   {wait for sport0 rx autobuffer interrupt}

NEWDATA:	 I3=I1;
		 MODIFY(I3,M2);   
		 AX0=DM(DATAIN);    {this routine sends the incoming}
                 DM(I3,M1)=AX0;     {a/d data straight to the d/a by}
                 AX0=DM(DATAIN+1);  {copying newly arrived data words from}
                 DM(I3,M1)=AX0;     {DATAIN into the DATAOUT buffer}
                 RTI;     

{The DATAOUT buffer is twice as long as the DATAIN buffer, since it contains}
{control words, as well as output data. This program ignores the control}
{words arriving back from the AD1849 during the data mode}       		    
                
.ENDMOD;
