.module/ram/abs=0       ad1847;

/*****************************************************************************
 ADSP-2101 -> AD1847: Talkthru Interface                      
                                                              
 This code provides the necessary initializations and setup to allow for 
 communication between the AD1847 SoundPort codec and the ADSP-2100 family 
 Serial Port 0.                
                                                              
 This interface code was written around an AD1847 connected to an ADSP-2101 
 EZ-Lab board through the J2 SPORT Connector.
 
 There are two modes of operation of this interface:          
                                                              
 1) If the board is RESET, the codec is initialized for an    
    8kHz sampling rate of stereo PCM data.                    
                                                              
 2) If the FLAG_IN button is held at reset, then the code enters its AD1847
    configuration mode.

 Codec attributes that can be changed are: line or microphone input source, 
 input gain, and the sample rate. Do not press the IRQ2 button too fast when 
 changing the sample rate as the codec needs time to perform its autoclibration
 procedure for each sampling rate. 

 Initialization: 
            SPORT1 must be configured as flags and interrupts. 
            irq2 must be enabled.

 Operation:
            Press and hold    <FLAG> button
            Press and release <IRQ2>  button
            Release           <FLAG> button to enter the setup routine

            Push <IRQ2> button to toggle between Line_1 and Line_2 input
            Push <FLAG> button to go to the next state

            Push <IRQ2> button to change the input gain, (8 levels)
            Push <FLAG> button to go to the next state

            Push <IRQ2> button to change the sample rate, (16 steps)
            Push <FLAG0> button to exit the setup routine


 Line Input Gain Default:       level 0  =  0dB
 
 Sample Rates:
            ( 1)      8 (default)     ( 9)     5.5125     
            ( 2)     16               (10)     11.025     
            ( 3)     27.42857         (11)     18.9
            ( 4)     32               (12)     22.05
            ( 5)     N/A              (13)     37.8
            ( 6)     N/A              (14)     44.1
            ( 7)     48               (15)     33.075
            ( 8)      9.6             (16)      6.615

 This module can be used without modification for the ADSP-2101, ADSP-2103, 
 and ADSP-2115. Use with the ADSP-2111, ADSP-2171, or ADSP-21msp5x processors
 would require modification of the interrupt vector table and any instructions
 relating to IFC, IMASK, or any other interrupt-related structure.                                 
                                                              
 This interface is not recommended for the ADSP-2105.       
*****************************************************************************/

.var/dm/ram/circ        rx_buf[3];      /* Status + L data + R data */
.var/dm/ram/circ        tx_buf[3];      /* Cmd + L data + R data    */
.init tx_buf:   0xc000, 0x0000, 0x0000; /* Initially set MCE        */


.var/dm/ram/circ        init_cmds[13];
/*********************************************************/
/*      Initial codec setup:                             */
/* 0:  Left Input Control: 0 gain, Line 1 input          */
/* 1:  Right Input Control: 0 gain, Line 1 input         */
/* 2:  Left Aux #1 Input Control: Muted                  */
/* 3:  Right Aux #1 Input Control: Muted                 */
/* 4:  Left Aux #2 Input Control: Muted                  */
/* 5:  Right Aux #2 Input Control: Muted                 */
/* 6:  Left DAC Control: 0 attenuation, muted            */
/* 7:  Right DAC Control: 0 attenuation, muted           */
/* 8:  Data Format: XTAL1, 8kHz sampling, stereo,        */
/*                 16-bit linear PCM                     */
/* 9:  Interface Config: Playback enabled, ACAL allowed  */
/* 10: Pin Control: CLKOUT active, XCTL1/0 LO            */
/* 12: Misc. Info: Transmit on 0,1,2, 32-word frame      */
/* 13: Digital Mix Control: DME Disabled, 0 attenuation  */
/*********************************************************/
  
.init init_cmds: 0xc000,  
		 0xc100,  
		 0xc280,  
		 0xc380, 
		 0xc480,  
		 0xc580,  
		 0xc680,  
		 0xc780, 
		 0xc850,  
		 0xc909,  
		 0xca00,  
		 0xcc40, 
		 0xcd00; 


.var/dm                 stat_flag;
.var/dm                 gain_state;
.var/dm                 sampling_state;
.var/dm                 chng_state;

reset_vect:     jump setup_mode; nop; nop; nop;

irq2_svc:       toggle FLAG_OUT; si = 1; dm(chng_state) = si; rti;

tx0_irq:        ar = dm(stat_flag);
		ar = pass ar;
		if eq rti; 
		jump next_cmd;

rx0_irq:        jump input_samples;
		rti; nop; nop;

		rti; nop; nop; nop;
		rti; nop; nop; nop;
		rti; nop; nop; nop;

/***************
   Code Start:
****************/
setup_mode:
                set flag_out;
                icntl = b#00100;               /* edge sensitive IRQ2      */
                ax0 = 0x0018;
                dm(0x3fff) = ax0;
                nop;
                nop;
                if FLAG_IN jump sys_init;

                ifc = b#000000111111;
                ax0 = 0;
                dm(chng_state) = ax0;
                
                imask = b#100000;
wait_irq:       ar = dm(chng_state);            /* wait for IRQ2 */
                ar = pass ar;
                if eq jump wait_irq;
                imask = 0x0000;

wait_rel_FO:    if NOT FLAG_IN jump wait_rel_FO; /* wait for FLAG_IN release */

                ax0 = 0;
                dm(chng_state) = ax0;

		jump set_codec;

sys_init:
		i0 = ^rx_buf;
		l0 = %rx_buf;
		i1 = ^tx_buf;
		l1 = %tx_buf;
		i3 = ^init_cmds;
		l3 = %init_cmds;

		m1 = 1;

/**********************
	sport0 setup: multichannel enable, ext. sclk, MFD=1,
			32 words, ext. rfs, DTYPE 00, 16 bits
**********************/ 
		ax0 = b#1000011000001111;
		dm(0x3ff6) = ax0;

/*********************
	Multichannel enable setup:
		Rx: 3, 4, 5, 19, 20, 21
		Tx: 0, 1, 2, 16, 17, 18
*********************/
		ax0 = b#0000000000000111;
		dm(0x3ff9) = ax0;
		dm(0x3ffa) = ax0;

		ax0 = b#0000000000000111;
		dm(0x3ff7) = ax0;
		dm(0x3ff8) = ax0;

/*********************
	SPORT Autobuffer Setup
		Rx: I0, M1
		Tx: I1, M1
**********************/
		ax0 = b#0000001010000111;
		dm(0x3ff3) = ax0;

		ax0 = b#0001000000000000;
		dm(0x3fff) = ax0;               /* SPORT0 Enabled,
						   SPORT1 = FI, FO, IRQs */
start_setup:    ifc = b#000000111111;
		nop;
		
                ax0 = 1;
                dm(stat_flag) = ax0;

		imask = b#010000;               /* enable tx0 interrupt */
		
		ax0 = dm(i1,m1);
		tx0 = ax0;
		
check_init:     ax0 = dm(stat_flag);            /* wait for entire init */
		af = pass ax0;                  /* buffer to be sent to */
		if ne jump check_init;          /* the codec            */

check_aci:      ax0 = dm(rx_buf);               /* once initialized, wait */
		ay0 = b#0000000000000010;       /* for codec to come out  */
		ar = ax0 and ay0;               /* of autocalibration     */
		if ne jump check_aci;

		idle;

		ay0 = 0xbf3f;
		ax0 = dm(init_cmds+6);
		ar = ax0 AND ay0;
		dm(tx_buf) = ar;               /* unmute left DAC */
		idle;

		ax0 = dm(init_cmds+7);
		ar = ax0 AND ay0;
		dm(tx_buf) = ar;               /* unmute right DAC */
		idle;

		ifc = b#000000111111;
		nop;
		imask = b#001000;       /* enable rx0 interrupt */

/************************************************************************/
/* Main Loop: talkthru                                                  */
/************************************************************************/

talkthru:       idle;
		jump talkthru;
		

/************************************************************************/
input_samples:
		ena sec_reg;
		ax1 = dm(rx_buf+1);     /* L channel input */
		mx1 = dm(rx_buf+2);     /* R channel input */
		dm(tx_buf+2) = mx1;
		dm(tx_buf+1) = ax1;
		rti;

next_cmd:
		ena sec_reg;
		ax0 = dm(i3,m1);
		dm(tx_buf) = ax0;
		ax0 = i3;
		ay0 = ^init_cmds;
		ar = ax0 - ay0;
		if gt rti;
		ax0 = 0x8000;
		dm(tx_buf) = ax0;
		ax0 = 0;               /* remove MCE if done initialization */
		dm(stat_flag) = ax0;   /* reset status flag */   
		rti;

/*****************************************************************************
 Codec configuration section
*****************************************************************************/

set_codec:      
                set FLAG_OUT;
                ax0 = 0;
                dm(gain_state) = ax0;
                dm(sampling_state) = ax0;
                

                IFC = b#000000111111;
                nop;
                imask = b#100000;                /* enable IRQ2 interrupt    */


/****************
 Input Selection
****************/

set_input:      if NOT FLAG_IN jump gain_wait;
                sr0 = dm(chng_state);
                af = pass sr0;
                if eq jump set_input;
                
                ax0 = dm(init_cmds);
                ay0 = 0xc080;
                ar = ax0 XOR ay0;       /* if already set for line 1 in,  */
                if ne jump set_to_2;    /* switch to line 2               */
                ax0 = 0xc000;
                dm(init_cmds) = ax0;
                ax0 = 0xc100;
                dm(init_cmds+1) = ax0;
                ax0 = 0;                /* reset flag for IRQ2 */
                dm(chng_state) = ax0;
                jump set_input;     

set_to_2:       ax0 = 0xc080;
                dm(init_cmds) = ax0;
                ax0 = 0xc180;
                dm(init_cmds+1) = ax0;                
                ax0 = 0;                /* reset flag for IRQ2 */
                dm(chng_state) = ax0;
                jump set_input;     

                                                
/****************
 Gain Selection
****************/
gain_wait:      if NOT FLAG_IN jump gain_wait;  /* software switch de-bounce */

set_gain:       if NOT FLAG_IN jump sampling_wait;                
                sr0 = dm(chng_state);
                af = pass sr0;
                if eq jump set_gain;

                ax1 = dm(gain_state);
                ar = pass ax1;
                if gt jump next1;

                ax0 = 0xfff0;           /* set gain = 0 for both inputs */
                ay0 = dm(init_cmds);
                ar = ax0 AND ay0;
                dm(init_cmds) = ar;
                ay0 = dm(init_cmds+1);
                ar = ax0 AND ay0;
                dm(init_cmds+1) = ar;
                ax0 = 1;
                dm(gain_state) = ax0;
                jump set_gain;


next1:          ax1 = dm(gain_state);
                ay0 = 1;
                ar = ax1 - ay0;
                if gt jump next2;

                ax0 = 0x0002;           /* set gain = 2 = 3dB */
                call gain_adjust;
                jump set_gain;

next2:          ax1 = dm(gain_state);
                ay0 = 2;
                ar = ax1 - ay0;
                if gt jump next3;

                ax0 = 0x0006;           /* set gain = 4 = 6dB */
                call gain_adjust;
                jump set_gain;

next3:          ax1 = dm(gain_state);
                ay0 = 3;
                ar = ax1 - ay0;
                if gt jump next4;

                ax0 = 0x0002;           /* set gain = 6 = 9dB */
                call gain_adjust;
                jump set_gain;

next4:          ax1 = dm(gain_state);
                ay0 = 4;
                ar = ax1 - ay0;
                if gt jump next5;

                ax0 = 0x000e;           /* set gain = 8 = 12dB */
                call gain_adjust;
                jump set_gain;

next5:          ax1 = dm(gain_state);
                ay0 = 5;
                ar = ax1 - ay0;
                if gt jump next6;

                ax0 = 0x0002;           /* set gain = 10 = 15dB */
                call gain_adjust;
                jump set_gain;

next6:          ax1 = dm(gain_state);
                ay0 = 6;
                ar = ax1 - ay0;
                if gt jump next7;

                ax0 = 0x0006;           /* set gain = 12 = 18dB */
                call gain_adjust;
                jump set_gain;

next7:          ax0 = 0x0002;           /* set gain = 13 = 21dB */
                call gain_adjust;
                ax0 = 0;
                dm(gain_state) = ax0;
                jump set_gain;

/*-------------
 gain adjust
------------*/
gain_adjust:
                ay0 = dm(init_cmds);
                ar = ax0 XOR ay0;
                dm(init_cmds) = ar;
                ay0 = dm(init_cmds+1);
                ar = ax0 XOR ay0;
                dm(init_cmds+1) = ar;
                ay0 = dm(gain_state);
                ar = ay0+1;
                dm(gain_state) = ar;
                ax0 = 0;
                dm(chng_state) = ax0;
                rts;

/************************
 Sampling Rate Selection
************************/
sampling_wait:  if NOT FLAG_IN jump sampling_wait;

set_sampling:   if NOT FLAG_IN jump done_config;
                sr0 = dm(chng_state);
                af = pass sr0;
                if eq jump set_sampling;
                
                ax0 = 0;
                dm(chng_state) = ax0;

set_rate:       ar = dm(sampling_state);
                ar = pass ar;
                if gt jump to_16;

to_8:           ar = 1;                  /* buffer initialized for 8 kHz */
                dm(sampling_state) = ar;
                jump set_sampling;


to_16:          ax0 = dm(sampling_state);
                ay0 = 1;
                ar = ax0 - ay0;
                if gt jump to_27;

                ax1 = 0xc852;                   /* set for 16.0 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 2;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_27:          ax0 = dm(sampling_state);
                ay0 = 2;
                ar = ax0 - ay0;
                if gt jump to_32;

                ax1 = 0xc854;                   /* set for 27.42857 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 3;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_32:          ax0 = dm(sampling_state);
                ay0 = 3;
                ar = ax0 - ay0;
                if gt jump to_na;

                ax1 = 0xc856;                   /* set for 32.0 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 4;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_na:          ax0 = dm(sampling_state);
                ay0 = 4;
                ar = ax0 - ay0;
                if gt jump to_na2;

                ax0 = 5;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_na2:         ax0 = dm(sampling_state);
                ay0 = 5;
                ar = ax0 - ay0;
                if gt jump to_48;

                ax0 = 6;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_48:          ax0 = dm(sampling_state);
                ay0 = 6;
                ar = ax0 - ay0;
                if gt jump to_96;

                ax1 = 0xc85c;                   /* set for 48.0 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 7;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_96:          ax0 = dm(sampling_state);
                ay0 = 7;
                ar = ax0 - ay0;
                if gt jump to_55;

                ax1 = 0xc85e;                   /* set for 9.6 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 8;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_55:          ax0 = dm(sampling_state);
                ay0 = 8;
                ar = ax0 - ay0;
                if gt jump to_11;

                ax1 = 0xc851;                   /* set for 5.5125 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 9;
                dm(sampling_state) = ax0;
                jump set_sampling;
                
to_11:          ax0 = dm(sampling_state);
                ay0 = 9;
                ar = ax0 - ay0;
                if gt jump to_18;

                ax1 = 0xc853;                   /* set for 11.025 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 10;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_18:          ax0 = dm(sampling_state);
                ay0 = 10;
                ar = ax0 - ay0;
                if gt jump to_22;

                ax1 = 0xc855;                   /* set for 18.9 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 11;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_22:          ax0 = dm(sampling_state);
                ay0 = 11;
                ar = ax0 - ay0;
                if gt jump to_37;

                ax1 = 0xc857;                   /* set for 22.05 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 12;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_37:          ax0 = dm(sampling_state);
                ay0 = 12;
                ar = ax0 - ay0;
                if gt jump to_44;

                ax1 = 0xc859;                   /* set for 37.8 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 13;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_44:          ax0 = dm(sampling_state);
                ay0 = 13;
                ar = ax0 - ay0;
                if gt jump to_33;

                ax1 = 0xc85b;                   /* set for 44.1 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 14;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_33:          ax0 = dm(sampling_state);
                ay0 = 14;
                ar = ax0 - ay0;
                if gt jump to_66;

                ax1 = 0xc85d;                   /* set for 33.075 kHz */
                dm(init_cmds+8) = ax1;
                ax0 = 15;
                dm(sampling_state) = ax0;
                jump set_sampling;

to_66:          ax0 = 0;
                dm(sampling_state) = ax0;

                ax1 = 0xc85f;                   /* set for 6.615 kHz */
                dm(init_cmds+8) = ax1;
                jump set_sampling;



done_config:    
                toggle FLAG_OUT;
                jump sys_init;
.endmod;
