{****************************************************************************
ADSP-2105 - AD1849(SOUNDPORT)  INTERFACE PROGRAM
Oli Josefsson, Kapriel Karagozyan, & Noam Levine     Analog Devices Jan, 1992

This program is written to run on an ADSP-2105.

For the hardware connections between the ADSP-2105 and AD1849 please refer
to the diagrams shown on the AD1849 data sheet.

Note: RFS1 and TFS1 MUST be tied together in order to initialize and 
operate the codec.

The AD1849 will be set up as follows:
  (NC indicates a no care state, all control reg are 1 bit unless indicated 
   by [#bits])

Control time slot control bits:
    DCB=0 followed by 1
    AC=1    autocalibrate
    DFR=5   Data conversion frequency,44.1kH [3]
    ST=1    stereo mode
    DF=0    dataformat is 16 bit twos complement [2]
    MCK=2   16.9344 MHz is the master clock [2]
    FSEL=0  frame size, 64 bits [2]
    MS=1    master mode (i.e. receive external serial clock)
    TXDIS=0 enable serial output
    ENL=0   disable loopback testing
    ADL=NC  loopback mode analog/digital (disabled)
    PIO=NC  paralell I/O bits not used [2]
    REVID=NC

Data Timeslot Control Bits:
    OM0=1    enable line 0 output
    OM1=1    enable line 1 output
    LO=0    no attenuation of left channel output [6]
    SM=0    mono output is muted
    RO=0    no attenuation of right channel output [6]
    PIO=NC  paralell I/O bits not used [2]
    OVR=NC  overrange INPUT
    IS=0    line-level stereo input selected
    LG=0    no gain for left channel [4]
    MA=15   no monitor mix (i.e. ADC output is not mixed with DAC 
input
    RG=0    no gain for right channel [4]

*****************************************************************************

In its current condition, this program can be used "as is" to perform straight
talkthrough (at 44.1 KHz sample rate) on both left and right channels of the 
AD1849. The incoming audio data from the 16bit ADCs is placed in a short buffer 
and immediately sent out to the 16bit DACs.

The initial setup and handshaking with the AD1849 starts with the "START"
routine. A state machine to perform the handshaking is executed in software.
Once the AD1849 is configured properly, the processor enters the WAIT_DATA
loop and waits for serial port interrupt requests.

This program is booted into the ADSP-2105 on power-up.

*****************************************************************************

Modified for use on DHD by Charles Steinkuehler on 10-9-92

*****************************************************************************}

.MODULE/ABS=0/BOOT=0  AD1849;

.CONST QLEN	=4096;

.VAR/CIRC  	CTRLIN[4];              {circular buffers for data input and}
.VAR/CIRC  	CTRLOUT[4];             {output for data mode and control mode}
.VAR/CIRC  	SERIN[4];		{SERIAL INPUT AUTOBUFFER}
.VAR/CIRC  	SEROUT[8];		{SERIAL OUTPUT AUTOBUFFER}
.VAR/DM/CIRC	RECDATA[QLEN];		{RECORD DATA QUEUE}
.VAR/DM/CIRC	PLAYDATA[QLEN];		{PLAYBACK DATA QUEUE}
.VAR       	FIRST_FLG;
.VAR       	DCB_FLG;
.VAR       	DMODE_FLG;
.var       	sync_flag;
.VAR/DM		PLAY_LEN;
.VAR/DM		REC_LEN;
.VAR/DM		PL_HOLD;
.VAR/DM		PR_HOLD;
.VAR/DM		RL_HOLD;
.VAR/DM		RR_HOLD;
.VAR/DM		RX_FULL;
.VAR/DM		TX_EMPT;

.port      mode_sel;            { latched control for Control / Data line }
.PORT	   PM_PAGE;		{ PM PAGE REG (USED AS EXT TEST SIGNAL) }
.VAR/DM	   DMA_REG0;		{ DMA DATA REG }
.VAR/DM	   DMA_REG1;		{ DMA STATUS REG }
.PORT	   DMA_REG2;		{ DMA STATUS REG }
.PORT	   DMA_REG3;		{ DMA STATUS REG }

           JUMP START;nop;nop;nop;         {restart interrupt}
           RTI;nop;nop;nop;                {IRQ2 int, not used}
           RTI;nop;nop;nop;                {SPORT0 tx not used}
           RTI;nop;nop;nop;                {SPORT0 rx not used}
           JUMP SETUPCONTROL;nop;nop;nop;  {SPORT1 transmit int}
           i3=i1;JUMP NEWDATA;nop;nop;     {SPORT1 receive int}
   {i3 -> tx pointer, must be set FIRST THING in interrupt routine }
       
           RTI;nop;nop;nop;                {timer int,not used}

START:  AX0=0; 
        dm(mode_sel)=AX0;    {set the AD1849 D/C pin low}
        M0=1;
        L0=%CTRLIN;
        I0=^CTRLIN;
        M1=1;
        L1=%CTRLOUT;
        I1=^CTRLOUT;
        AX0=1;
        DM(FIRST_FLG)=AX0;	
        DM(DCB_FLG)=AX0;	
        AX0=0;
        DM(DMODE_FLG)=AX0;      { in control mode }
        AY0=B#0010000100101100; 

{Initialize contol mode output buffer}
                 
           DM(I1,M1)= B#0010000100101100; 
{DCB=0,AC=1,DFR=05,ST=1,DF=00}
           DM(I1,M1)= B#0010001000000000; {MCK=02,FSEL=0,MS=1,TXDIS=0}
                                          {ENL=0,ADL=0}
           DM(I1,M1)= B#0000000000000000; {PIO = 00}
           DM(I1,M1)= B#0000000000000000; {REVID=NC}
           L2 = 0;                  { linear addressing for register }
           I2 = 0x3fef;             { point to last DM cntrl reg }

{3FEF}     DM(I2,M1) = 0x0283;      { sport1 autobuffer register }
{3FF0}     DM(I2,M1) = 383;         { rfsdiv1	}
{3FF1}     DM(I2,M1) = 849;         { sclkdiv1 	}
{3FF2}     DM(I2,M1) = B#0100000100011111;  { sport1 control register:                                                      
                                              internal sclk & rfs                                                             
                                              normal framing mode                                                             
                                              frame sync not inverted
                                              16-bit word length }

           I2=0x3ffb;

{3FFB}     DM(I2,M1) = 0x0000;      { TSCALE register }
{3FFC}     DM(I2,M1) = 0x0000;      { TCOUNT register }
{3FFD}     DM(I2,M1) = 0x0000;      { TPERIOD register,initializing value
                                      for TCOUNT after every interrupt }

{3FFE}     DM(I2,M1) = 0x0000;      { external data memory waits=0 }


           ICNTL=0x00 ;
           IMASK=B#000100;          { only SPORT1 tx interurpt enabled
                                      initially while in control mode }
 
{.....Set bit test mask for DCB bit, used in tx interrupt state machine....}

     AY0=B#0010000100101100; { test mask for DCB bit } 

{.....       send first control word to switch codec to data mode       .....}
           AX0=DM(I1,M1);       { send first 16bits of ctrl word }                         
           TX1=AX0;

{3FFF}     DM(I2,M1) = 0x0c18;  { system control reg: sport1 enabled }

{..... Wait for an interrupt indicating that transmit register is ready for 
       new data and that the 2105 has received a 16bit word             .....}

WAIT1:     AX1=DM(DMODE_FLG);   { check dmode flag }
           AR=PASS AX1;
           IF GT JUMP GO_DMODE; { if set, in data mode }
           JUMP WAIT1;          { else, wait for initialization to be completed                                              
                                  from tx interrupt routine }

GO_DMODE:  L0=%SERIN;          { init I0, L0 for rx autobuffer }
           I0=^SERIN;
           L1=%SEROUT;         { init I1, L1 for tx autobuffer }

           I1=^SEROUT;
           M3=3;
           L3=L1;

	   L2=QLEN;
	   M2=1;
   	   M4=1;
	   L4=QLEN;
	   I4=^RECDATA;
	   M5=1;
	   L5=QLEN;
	   I5=^PLAYDATA;
	   M6=1;
	   L6=QLEN;
	   I6=^RECDATA;
           M7=1;
	   L7=QLEN;
	   I7=^PLAYDATA;
	   AX1=QLEN;		{ SET REG FOR QUICK LENGTH CHECK }
	   AR=PASS 0;	   	{ ZERO LENGTH REGS }
	   DM(PLAY_LEN)=AR;
	   DM(REC_LEN)=AR;
	   ENA SEC_REG;		{ INTERRUPT REG SET }
	   AX1=QLEN;		{ SET REG FOR QUICK LENGTH CHECK }
	   DIS SEC_REG;	
	   AR=PASS 0;
	   DM(DMA_REG0)=AR;	{ INIT PHONY DMA DATA }
	   AR=PASS 1;		
	   DM(DMA_REG1)=AR;	{ INIT PHONY DMA STATUS }

           AX0=DM(I1,M1);         { send first 16bits of data }
           TX1=AX0;
           AX0=0X0c18;
           DM(0X3FFF)=AX0;        { turn on sport1 }

           IFC=B#000000111111;    { clear all pending interrupts }
           nop;                   { cycle for IFC latency }
           IMASK=B#000010;        { sport1 rx interrupt on }

DATA_LOOP: 
	   AY0=DM(REC_LEN);		{ GET REC QUEUE LENGTH }
	   DM(DMA_REG2)=AY0;		{ PUT LEN IN SHARED REG 2 }
	   AY0=DM(PLAY_LEN);		{ GET PLAY QUEUE LENGTH }
	   DM(DMA_REG3)=AY0;		{ PUT LEN IN SHARED REG 3 }
	   AR=PASS 0;
	   DM(PM_PAGE)=AR;		{ ZERO EXT VISIBLE INT FLAG }
	   AX1=QLEN;			{ SET REG FOR QUICK LENGTH CHECK }
TX:
	   AX0=DM(TX_EMPT);		{ GET TX FLAG }
	   AR=PASS AX0;
	   IF EQ JUMP RX;		{ IF FULL SKIP }
	   AY0=DM(PLAY_LEN);
	   AX0=2;
	   AR=AY0-AX0;	      		{ NEW LEN = LEN-2 }
	   IF NOT AC JUMP RX; 		{ IF BORROW -> SKIP..NO DATA }
	   AX0=DM(I5,M5);		{ READ LEFT DATA }
	   DM(PL_HOLD)=AX0;		{ LEFT DATA TO HOLD REG }
	   AX0=DM(I5,M5);		{ READ RIGHT DATA }
	   DM(PR_HOLD)=AX0;		{ RIGHT DATA TO HOLD REG }
	   DM(PLAY_LEN)=AR;		{ WRITE NEW LEN }
	   AR=PASS 0;
	   DM(TX_EMPT)=AR;		{ CLEAR TX FLAG }
RX:
	   AX0=DM(RX_FULL);		{ GET RX FLAG }
	   AR=PASS AX0;
	   IF EQ JUMP PLAY;		{ IF EMPTY SKIP }
	   AY0=DM(REC_LEN);
	   AX0=2;
	   AF=AX0+AY0;	      		{ NEW LEN = LEN+2 }
	   AR=AX1-AF;			{ CHECK IF NEW LEN > BUF LEN }
	   IF NOT AC JUMP PLAY;    	{ IF BORROW -> SKIP..NO DATA }
	   AX0=DM(RL_HOLD);		{ READ LEFT DATA }
	   DM(I4,M4)=AX0;		{ LEFT DATA TO QUEUE }
	   AX0=DM(RR_HOLD);		{ READ RIGHT DATA }
	   AR=PASS AF,			{ NEW PLAY LEN TO AR }
	   DM(I4,M4)=AX0;		{ RIGHT DATA TO QUEUE }
	   DM(REC_LEN)=AR;		{ WRITE NEW LEN }
	   AR=PASS 0;
	   DM(RX_FULL)=AR;		{ CLEAR RX FLAG }
PLAY:
	   AX0=DM(DMA_REG1);		{ READ DMA STATUS WORD }
	   AY1=0X0100;			{ MASK FOR FULL BIT }
	   AR=AX0 AND AY1;		{ TEST FOR FULL (DMA DATA IN) }
	   IF EQ JUMP REC;		{ IF NO DATA GOTO REC }
	   AY0=DM(PLAY_LEN);		{ GET QUEUE LENGTH }
	   AR=AX1-AY0;			{ COMP QUEUE LEN WITH BUF SIZE }
	   IF NOT AC JUMP REC;		{ IF FULL GOTO REC }
	   AX0=DM(DMA_REG0);		{ IF ROOM, READ DATA FROM DMA }
	   AR=AY0+1,			{ INC QUEUE LENGTH }
	   DM(I7,M7)=AX0;		{ WRITE DATA TO PLAY QUEUE }
	   DM(PLAY_LEN)=AR;	    	{ WRITE NEW LENGTH }
	   AX0=0X0001;	    		{ UPDATE 'FAKE' STATUS REG }
	   DM(DMA_REG1)=AX0;
REC:
	   AX0=DM(DMA_REG1);		{ READ DMA STATUS WORD }
	   AY1=0X0001;			{ MASK FOR EMPTY BIT }
	   AR=AX0 AND AY1;		{ TEST FOR EMPTY (DMA OUT READY) }
	   IF LE JUMP DATA_LOOP;	{ IF NOT EMPTY LOOP } 
	   AY0=DM(REC_LEN);		{ GET QUEUE LENGTH }
	   AR=PASS AY0;			{ CHECK SIZE FOR EMPTY }
	   IF EQ JUMP DATA_LOOP;	{ IF NO DATA LOOP }
	   AR=AY0-1,			{ DEC QUEUE LENGTH }
	   AX0=DM(I6,M6);		{ READ DATA FROM QUEUE }
	   DM(DMA_REG0)=AX0;		{ WRITE DATA TO DMA }
	   DM(REC_LEN)=AR;		{ WRITE NEW LENGTH }
	   AX0=0X0100;	    		{ UPDATE 'FAKE' STATUS REG }
	   DM(DMA_REG1)=AX0;
	   JUMP DATA_LOOP;   	{DO UNTIL sport1 rx autobuffer interrupt}


{............................................................................}
{............................................................................}
{ sport1 tx interrupt routine	                                               }
{ -------------------------------------------------------------------------- }
{ This routine initializes the AD1849 control mode and then waits to see     }
{ if the codec is ready to be switched to data mode. The routine also        }
{ initializes the transmit autobuffer with the appropriate data-mode         }
{ control words. See the AD1849 data sheet for a complete explanation        }
{ of control word bits.                                                      }
{	                                                                       }
{ Note: AY0 contains a bit mask and must NOT be modified elsewhere           }
{............................................................................}
{............................................................................}

SETUPCONTROL:    AX0=DM(FIRST_FLG);   { first time through ? }                           
         AF=PASS AX0;
         IF NE JUMP DECR_FIRST;  { if so, wait until next word transmitted}
         AX0=DM(DCB_FLG); 
         AR=PASS AX0;
         IF EQ JUMP DCBFLG_SET;
         AX0=DM(CTRLIN);         {DCB_FLG has not been set yet}
         AR=AX0 XOR AY0;         {check all incoming bits including DCB bit}
         IF EQ JUMP SET_DCB;     {set flag if DCB was 0}
         RTI;

DCBFLG_SET:  AX0=DM(CTRLIN);     {DCB_FLG was set}
         AR=AX0 AND AY0;         {only check for DCB bit}
         IF NE JUMP SETDMODE;    {if DBC=1 ready for datamode}
         RTI;

SET_DCB: AX0=0;
         DM(DCB_FLG)=AX0;
         AX0=B#0010010100101100; {DCB was 0, prepare to send DCB=1}
         DM(CTRLOUT)=AX0;
         AY0=B#0000010000000000; 
         RTI;

DECR_FIRST:  AX0=0;
         DM(FIRST_FLG)=AX0;     { if first time, set flag=0 }
         RTI;

SETDMODE:        IMASK=0;
                 AX0=0X0418;              {disable sport1}
                 DM(0X3FFF)=AX0;

{ At this point we could boot page#1 and do the following data mode setup 
  after the reboot occurs.  This would free up some PM RAM for other uses.  
  
  The data mode setup initializes the transmit autobuffer control word elements. 
These values are not changed in this talkthrough application.    }
                 
                 I1 = ^SEROUT;
                 L1=0;
                 DM(I1,M1) = 0x0000;       {reset output & input data buffers}
                 DM(I1,M1) = 0x0000;       {initialize embedded control bits}
                 DM(I1,M1) = B#1100000000000000; {OM1=1,OM1=1,LO=0,SM=0,RO=0}
                 DM(I1,M1) = B#0000000011110000; 
                                           {PIO=00,OVR=0,IS=0,LG=0,MA=15,RG=0}
                 DM(I1,M1) = 0x0000;       {reset output & input data buffers}
                 DM(I1,M1) = 0x0000;       {initialize embedded control bits}
                 DM(I1,M1) = B#1100000000000000; 
                                           {OM1=1,OM1=1,LO=0,SM=0,RO=0}
                 DM(I1,M1) = B#0000000011110000;
                                           {PIO=00,OVR=0,IS=0,LG=0,MA=15,RG=0}
                 AX0=0X001F;
                 DM(0X3FF2)=AX0;           { sport1 control:              
                                             internal tfs
                                             external sclk & rfs
                                             16 bit words }
                 AX0=1;
                 dm(mode_sel)=AX0;         { set D/C high }
                 DM(DMODE_FLG)=AX0;        { set data mode flag high }
                 RTI;

{............................................................................}
{............................................................................}
{ sport1 rx interrupt routine                                                }
{ -------------------------------------------------------------------------- }
{ This routine sends the incoming a/d data straight to the d/a by            }
{ copying newly arrived data words from the SERIN buffer into the           }
{ SEROUT buffer	                                                           }
{............................................................................}
{............................................................................}
 
NEWDATA:  
	  ENA SEC_REG;		{ USE SECONDARY REGS }
	  AR=PASS 1;		
	  DM(PM_PAGE)=AR;	{ SET EXT VISIBLE INT SIGNAL }
	  MODIFY (I3,M3);	{ POINT TO DATA }
	  AX0=DM(PL_HOLD);	{ READ DATA FROM HOLDING REG}
          DM(I3,M1)=AX0;    	{ output LEFT channel data }
	  AX0=DM(PR_HOLD);	{ READ DATA FROM HOLDING REG}
          DM(I3,M1)=AX0;    	{ output RIGHT channel data } 
	  AX0=DM(SERIN);   	{ get LEFT channel data } 
	  DM(RL_HOLD)=AX0;	{ PUT LEFT CH IN HOLDING REG }
          AX0=DM(SERIN+1); 	{ get RIGHT channel data } 
	  DM(RR_HOLD)=AX0;	{ PUT RIGHT CH IN HOLDING REG }
	  AR=PASS 1;		{ LEN TO AR }
	  DM(RX_FULL)=AR;	{ SET RX FULL FLAG }
	  DM(TX_EMPT)=AR;	{ SET TX EMPTY FLAG }

	  AR=PASS 0;	 	{ ZERO PLAY REGS }
	  DM(PL_HOLD)=AR;	
	  DM(PR_HOLD)=AR;

{............................................................................}
{ Error checking section (optional). Incoming control words are compared     }
{ to the outgoing control words.                                             }
{............................................................................}

TESTERR:
         ax0=dm(SEROUT+2);  { read known output control word }
         ay0=dm(SERIN+2);   { read newly received control word }
         ar=ax0-ay0;         { compare, they should be the same }
         if eq jump no_error;{ if same, no error }
         ax0=dm(sync_flag);  { if not, read SYNC_FLAG }
         ar=pass ax0;        { test for 0 or 1 }
         if ne jump reset_sport;{ if 1, second failure, reset SPORT1 }

         ar=pass 1;          { else, first failure, set SYNC_FLAG }
         dm(sync_flag)=ar;	
	 DIS SEC_REG;
         rti;                { return }

reset_sport:
         si=0x418;           { disable SPORT1 }
         dm(0x3fff)=si;		
         si=0xc18;           { re-enable SPORT1 }
         dm(0x3fff)=si;

no_error: ar=pass 0;
         dm(sync_flag)=ar;   { reset SYNC_FLAG }
	 DIS SEC_REG;
         rti;                { return }

{ -------------------------------------------------------------------------- }
{ The SEROUT buffer is twice as long as the SERIN buffer, since it         }
{ contains control words, as well as output data. This program ignores the   }
{ control words arriving back from the AD1849 during the data mode, except   }
{ for the purpose of error checking.                                         }
{ -------------------------------------------------------------------------- }
               
.ENDMOD;


