{****************************************************************************
ADSP-2105 - AD1849(SOUNDPORT)  INTERFACE PROGRAM
Oli Josefsson, Kapriel Karagozyan, & Noam Levine     Analog Devices Jan, 1992

This program is written to run on an ADSP-2105.

For the hardware connections between the ADSP-2105 and AD1849 please refer
to the diagrams shown on the AD1849 data sheet.

Note: RFS1 and TFS1 MUST be tied together in order to initialize and 
operate the codec.

*****************************************************************************

The initial setup and handshaking with the AD1849 starts with the "START"
routine. A state machine to perform the handshaking is executed in software.
Once the AD1849 is configured properly, the processor enters the WAIT_DATA
loop and waits for serial port interrupt requests.

*****************************************************************************

Modified for use on DHD by Charles Steinkuehler on 10-9-92
Modified for DHD2A by Charles Steinkuehler on 12-27-93
Modified for DHD3 by Charles Steinkuehler on 3-12-94
Modified for DHD3 by Marty Flickinger on 7-15-94

  Single direction DMA channel with direction flag port
  Updated to current ADI 1849 interface code
   -equal length transmit and receive circular buffers
   -direct addressing of transmit buffer
   -this frees one index register (I3) and prevents problems due to clk rate

*****************************************************************************}

.MODULE/ABS=0/BOOT=0  AD1849;

.CONST CHANMASK	= B#00000111;	{ 4 Channels }
.CONST MODEMASK	= B#00001000;	{ Xfer mode }
.CONST LENMASK	= B#11110000;	{ 15 Blocks max xfer/16 registers }
.CONST ENMASK	= B#00000001;	{ 1 = Enabled, 0 = Disabled }
.CONST DIRMASK	= B#00000010;	{ 1 = DSP to DMA, 0 = DMA to DSP }
.CONST FULLMASK	= B#00010000;	{ 1 = DMA reg full, 0 = DMA reg empty }

.CONST CTRLWD0	= B#0010000000010100; {DCB=0,AC=0,DFR=2,ST=1,DF=00}
.CONST CTRLWDCB	= B#0010010000010100; {DCB=1,AC=0,DFR=2,ST=1,DF=00}
.CONST CTRLWD1	= B#0001001000000000; {MCK=1,FSEL=0,MS=1,TXDIS=0,ENL=0,ADL=0}
.CONST CTRLWD2	= B#0000000000000000; {PIO = 00}
.CONST CTRLWD3	= B#0000000000000000; {REVID=NC}
.CONST DCBMASK	= B#0000010000000000; {DCB MASK}

.CONST DATAWD0	= B#1100000000000000; {OM0=1,OM1=1,LO=0,SM=0,RO=0}
.CONST DATAWD1	= B#0000000011110000; {PIO=00,OVR=0,IS=0,LG=7,MA=15,RG=7}

.CONST BUFLEN	= 0x0400;	{ 4 DMA blocks - 1024 Words }
.CONST TBUFLEN	= 0x0800;	{ Size for double buffering }

.CONST CH0BASE	= TBUFLEN * 0;
.CONST CH1BASE	= TBUFLEN * 1;
.CONST CH2BASE	= TBUFLEN * 2;
.CONST CH3BASE	= TBUFLEN * 3;

.CONST NUMCHANS		= 4;


.VAR/CIRC	CTRLIN[4];	{ Circular buffers for data I/O for }
.VAR/CIRC	CTRLOUT[4];	{ data mode and control mode }

.VAR/CIRC	SERIN[4];	{ SERIAL INPUT AUTOBUFFER }
.VAR/CIRC	SEROUT[4];	{ SERIAL OUTPUT AUTOBUFFER }

.VAR		FirstFlag;	{ Forces 2 sends }
.VAR		DCBflag;	{ C/D transition helper }
.VAR		DataMdFlg;	{ Codec in data mode? }
.VAR		SyncFlag;	{ Detects 2 consecutive serial errors }

.VAR		RecFlag;	{ Direction bit 1=Rec,0=Play }
.VAR		DMArec;		{ Direction bit 1=Rec,0=Play }
.VAR		Field;		{ Field bit (toggles) }
.VAR		CodecBuff;	{ Ptr to start of serial CODEC buffer }
.VAR		StatSave;	{ ASTAT register interrupt save location }
.VAR		ZonkOVR;	{ Counter for asserting OVR=0 }
.VAR		FieldSwitch;	{ Indicates field switch occurred }
.VAR		NewParam;	{ Some parameter changed }
.VAR		StatBits;	{ DMA status and misc flags (port mirror) }
.CONST FLG_BUSY	= B#00000001;	{ Busy doing DMA xfer }
.CONST FLG_DONE	= B#00000010;	{ Reg write/DMA done }
.CONST FLG_CLIP	= B#10000000;	{ Input clipping }

{.VAR		PlayLen;}	{ Header length value for play data }
.VAR		ChanPtrs[4];	{ Ptr to start of DMA bufs, 1 per chan }

.CONST	NUMARGS = 2;
.VAR/PM/CIRC	ChanArgs[NUMCHANS*NUMARGS];	{ Channel parameters }
.VAR/PM/CIRC	ChanArgsSync[NUMCHANS*NUMARGS];	{ Channel parameters to use }
.CONST	ARG_Volume	= 0;
.CONST	ARG_Pan		= 1;

.CONST	NUMGLOBS = 3;
.VAR		RegArray[NUMGLOBS];	{ User-accessible global registers }
.CONST	GLOB_OutAtten	= 0;		{ Left/Right output attenuation }
.CONST	GLOB_InputGain	= 1;		{ Left/Right input gain }
.CONST	GLOB_Flags	= 2;		{ Global flags }
.CONST		GFLG_DIR = B#00000001;

.port		mode_sel;	{ I_O reg for A/D control/data mode pin }
.port		dmastat;	{ Dma status port }
.port		dmadata;	{ Dma data port }
.port		XferCtrl;	{ R=BlkLen/reg,mode,channel;W=DMA Status }
.port		RegDataLo;	{ R=Register Data/W=Not Used }
.port		RegDataHi;	{ Channel enable bits }


	{ Vector Table }

	JUMP START;nop;nop;nop;		{ Restart interrupt }
	JUMP VertInt;nop;nop;nop;	{ IRQ2 int, vertical rate }
	RTI;nop;nop;nop;		{ SPORT0 Tx not used }
	RTI;nop;nop;nop;		{ SPORT0 Rx not used }
	JUMP SETUPCONTROL;nop;nop;nop;	{ SPORT1 transmit int }
	JUMP NEWDATA;nop;nop;nop;	{ SPORT1 receive int }
	RTI;nop;nop;nop;		{ Timer int,not used }

START:
	ENA M_MODE;			{ Integer multiplies }

	AX0=0; 
	dm(mode_sel)=AX0;		{ AD1849 D/C pin low = control mode }
	M0=1;
	L0=%CTRLIN;
	I0=^CTRLIN;
	M1=1;
	L1=%CTRLOUT;
	I1=^CTRLOUT;
	AX0=1;
	DM(FirstFlag)=AX0;	
	DM(DCBflag)=AX0;	
	AX0=0;
	DM(DataMdFlg)=AX0;		{ In control mode (not data mode) }
	DM(ZonkOVR) = ax0;
	DM(FieldSwitch) = ax0;
	DM(NewParam) = ax0;

{ Initialize contol mode output buffer }
	DM(I1,M1) = CTRLWD0;
	DM(I1,M1) = CTRLWD1;
	DM(I1,M1) = CTRLWD2;
	DM(I1,M1) = CTRLWD3;

	L2 = 0;			{ Linear addressing for register }
	I2 = 0x3fef;		{ Point to SPORT1 control registers }

{3FEF}	DM(I2,M1) = 0x0283;	{ SPORT1 autobuffer: Tx I1,M1  Rx I0,M0 }
{3FF0}	DM(I2,M1) = 383;	{ RFSDIV1 }
{3FF1}	DM(I2,M1) = 849;	{ SCLKDIV1 }
{3FF2}	DM(I2,M1) = B#0100000100011111;	{ SPORT1 control register:
						Internal SCLK & RFS
						normal framing mode
						frame sync not inverted
						16-bit word length }

	I2=0x3ffb;

{3FFB}	DM(I2,M1) = 0x0000;	{ TSCALE register }
{3FFC}	DM(I2,M1) = 0x0000;	{ TCOUNT register }
{3FFD}	DM(I2,M1) = 0x0000;	{ TPERIOD register,initializing value
					for TCOUNT after every interrupt }

{3FFE}	DM(I2,M1) = 0x0000;	{ External data memory waits=0 }

	ICNTL=0x00;
	IMASK=B#000100;		{ Only SPORT1 Tx interrupt enabled
					initially while in control mode }
 
{.....Set bit test mask for DCB bit, used in Tx interrupt state machine....}

	AY0 = CTRLWD0;		{ Test mask for DCB bit } 

{.....Send first control word to switch codec to data mode.....}
	AX0=DM(I1,M1);		{ Send first 16bits of ctrl word }
	TX1=AX0;

{3FFF}	DM(I2,M1) = 0x0c18;	{ System control reg: SPORT1 enabled }

{.....Wait for an interrupt indicating that transmit register is ready for 
	new data and that the 2105 has received a 16bit word.....}

WAIT1:	AX1=DM(DataMdFlg);	{ Check dmode flag }
	AR=PASS AX1;
	IF GT JUMP GO_DMODE;	{ If set, in data mode }
	JUMP WAIT1;		{ Else, wait for init to be completed
					from tx interrupt routine }

GO_DMODE:
	L0=%SERIN;		{ Init I0, L0 for Rx autobuffer }
	I0=^SERIN;
	M0=1;
	L1=%SEROUT;		{ Init I1, L1 for Tx autobuffer }
	I1=^SEROUT;
	M1=1;

	{ DAG 2: CODEC queue }
	I2 = CH0BASE;
	L2 = 0;
	M2 = 1;
	dm(CodecBuff) = I2;

	{ DAG 3: DMA queue }
	I3 = CH0BASE + BUFLEN;
	L3 = BUFLEN;
	M3 = 1;

	{ DAG 4: Channel Gain Coefficents }
	I4 = ^ChanArgsSync+ARG_Volume;
	L4 = %ChanArgsSync;
	M4 = NUMARGS;

	{ DAG 5 & 6: IRQ Indirect access }
	L5 = 0;
	M5 = 1;
	L6 = 0;
	M6 = 1;

	{ DAG 7: Non-IRQ Indirect access }
	L7 = 0;
	M7 = 1;

	{ Zero all volume coefficients }
	cntr = NUMCHANS;
	ax0 = 0;
	I7 = ^ChanArgs;
	DO ZeroCoef UNTIL ce;
	pm(I7,M4) = ax0;
ZeroCoef: pm(I4,M4) = ax0;

	{ Init default global args }
	I7 = ^RegArray;
	DM(I7,M7) = 0x0000;	{ Left output attenuation }
	DM(I7,M7) = 0x0000;	{ Right output attenuation }
	DM(I7,M7) = 0x0007;	{ Left input gain }
	DM(I7,M7) = 0x0007;	{ Right input gain }
	DM(I7,M7) = GFLG_DIR;	{ Flags }

	{ Initalize variables }
	ar = PASS 0;
	dm(Field) = ar;
{	dm(stat)= ar;}
	ax0 = 1;
	dm(RecFlag) = ax0;	{ Dir = record }

	{ Initalize Channel pointers }
	ax0 = CH0BASE + BUFLEN;
	dm(ChanPtrs + 0) = ax0; 
	ax0 = CH1BASE + BUFLEN;
	dm(ChanPtrs + 1) = ax0; 
	ax0 = CH2BASE + BUFLEN;
	dm(ChanPtrs + 2) = ax0; 
	ax0 = CH3BASE + BUFLEN;
	dm(ChanPtrs + 3) = ax0; 


	AX0=DM(I1,M1);		{ Send first 16bits of data }
	TX1=AX0;
	AX0=0x0c18;
	DM(0x3FFF)=AX0;		{ Turn on SPORT1 }

	IFC=B#000000111111;	{ Clear all pending interrupts }
	nop;			{ Cycle for IFC latency }
	ICNTL=B#00100;		{ IRQ2 edge sensitive, int nesting off }
	IMASK=B#100010;		{ IRQ2 & SPORT1 Rx interrupt on }

	{ Set initial status bits (mirror register and port) }
	ax0 = 0;
	dm(StatBits) = ax0;
	dm(XferCtrl) = ax0;	{ Write Dma status reg }

DataLoop: 
	{ Wait for new enable }
	ay0 = ENMASK;
	ax0 = dm(dmastat);
	DO Enable UNTIL ne;
	ar  = dm(FieldSwitch);
	ar  = PASS ar;
	if ne CALL NewField;
	ar  = ax0 and ay0;
Enable:	ax0 = dm(dmastat);

	{ Use hardware direction bit for dma transfer direction }
	ay0 = DIRMASK;
	ar = ax0 and ay0;
	dm(DMArec) = ar;

	{ Get channel,blklen/reg/xfer mode bit }
	ax0 = dm(XferCtrl);

	{ Update status to "busy" }
	ax1 = dm(StatBits);
	ay0 = FLG_BUSY;
	ar  = ax1 OR ay0;
	dm(StatBits) = ar;
	dm(XferCtrl) = ar;	{ Write Dma status reg }

	{ Get xfer mode bit }
	ay0 = MODEMASK;
	ar  = ax0 and ay0;
	if eq jump dodma;

	{ Concatenate both 8 bit registers into user's word }
	ay0 = 0x00FF;
	ax1 = dm(RegDataHi);	{ Read user's data }
	ar  = ax1 AND ay0;
	sr  = LSHIFT ar BY 8 (LO);	{ Put in top 8 bits }
	ax1 = dm(RegDataLo);	{ Read user's data }
	ar  = ax1 AND ay0;
	sr  = sr OR LSHIFT ar BY 0 (LO);	{ Put in low 8 bits }
	ax1 = sr0;

	{ Get register number to write }
	ay0 = LENMASK;
	ar  = ax0 and ay0;	{ Mask for register number }
	sr  = LSHIFT ar BY -4 (LO);	{ Convert to 0-F }

	ay0 = NUMARGS;
	ar  = sr0 - ay0;	{ Global value or channel-specific? }
	if ge jump globals;

	{ Get channel number spec'd }
	ay0 = CHANMASK;
	ar  = ax0 AND ay0;	{ Determine channel selected for DMA }

	my0 = NUMARGS;
	mr  = ar * my0 (UU);	{ Make index into channel args }
	ay0 = sr0;
	ar  = mr0 + ay0;	{ Add it into index }

	ay0 = ^ChanArgs;	{ Address of channel registers }
	ar  = ar + ay0;		{ Add global register to array ptr }
	I7  = ar;		{ Use as an address }

	pm(I7,M7) = ax1;	{ Store value in register }
	jump ArgEnd;


globals:
	ay0 = ^RegArray;	{ Address of registers }
	ar  = ar + ay0;		{ Add global register to array ptr }
	I7  = ar;		{ Use as an address }
	dm(I7,M7) = ax1;	{ Store value in register }
ArgEnd:
	ax1 = 1;
	DM(NewParam) = ax1;
	jump Finish;

dodma:
	{ ax0 = channel and blocklen }
	{ Calculate buffer pointer }
	ay0 = CHANMASK;
	ar  = ax0 and ay0;	{ Determine channel selected for DMA }
	ay0 = ^ChanPtrs;	{ Address of buffer address array }
	ar  = ar + ay0;		{ Add channel to array start }
	I7  = ar;		{ Use as an address }
	ax1 = dm(I7,M7);	{ Read in buffer start address }
	I3  = ax1;		{ Use buffer address for DMA DAG }
	
	{ Calculate buffer length }
	ay0 = LENMASK;
	ar  = ax0 and ay0;	{ Mask for length bits }
	sr  = LSHIFT ar BY 4 (LO);	{ Shift to calculate word count }
	ax1 = sr0;		{ Result to ax1 }
{	dm(len) = sr0;}

	{ Check for zero length }
	ar  = PASS ax1;
	if eq jump Finish;	{ Skip, if length = 0 }

	ay0 = FULLMASK;		{ Setup for quick test }

	{ Branch to In or Out }
	ay1 = dm(DMArec);		
	ar  = PASS ay1;
	if eq jump DmaIn;

DmaOut:	cntr = ax1;
	ax1 = dm(I3,M3);
	DO EndOut UNTIL ce;
	ax0 = dm(dmastat);
	DO Empty Until eq;
	ar  = ax0 and ay0;	{ And with Fullmask }
Empty:	ax0 = dm(dmastat);	{ Read dma hardware status register }
	dm(dmadata) = ax1;	{ Write to DMA }
EndOut:	ax1 = dm(I3,M3);	{ Read next word to transfer }

	jump Finish;

DmaIn:	
	{ Read first word - Length }
	ax0 = dm(dmastat);
	DO Full1 UNTIL ne;
	ar  = ax0 and ay0;
Full1:	ax0 = dm(dmastat);
	ay1 = dm(dmadata);	{ Read from DMA	}
	dm(I3,M3) = ay1;	{ Write to buffer }
{	dm(PlayLen)  = ay1;}

	{ Read second word - Pad }
	ax0 = dm(dmastat);
	DO Full2 UNTIL ne;
	ar  = ax0 and ay0;
Full2:	ax0 = dm(dmastat);
	ax0 = dm(dmadata);	{ Read from DMA	}
	dm(I3,M3) = ax0;	{ Write to buffer }

	{ Test for valid len > xfer len }
	ar = ax1 - ay1;
	if ge jump WdLen;	{ If exact len < xfer len OK }
	ay1 = ax1;		{ Else exact len = xfer len }

WdLen:	{ Calculate valid word length }
	ax0 = 2;
	ar  = ay1 - ax0;	{ Word length - headerlen valid words }

	{ Transfer all valid data words }
	cntr = ar;
	DO EndVld UNTIL ce;
	ax0 = dm(dmastat);
	DO Full3 UNTIL ne;
	ar  = ax0 and ay0;
Full3:	ax0 = dm(dmastat);
	ax0 = dm(dmadata);	{ Read from DMA	}
EndVld:	dm(I3,M3) = ax0;	{ Write to buffer }

	{ Calculate remaining number of samples }
	ar = ax1 - ay1;		{ Subtract exact len from xfer len }
	sr = LSHIFT ar BY -1 (LO);

	{ If no more samples, skip to Finish }
	ar = PASS sr0;
	if eq jump Finish;

	{ Read last valid data words }
	M3  = -2;
	modify (I3,M3);		{ Point to last valid data words }
	M3  = 1;
	ax1 = dm(I3,M3);	{ Read Left data }
	ay1 = dm(I3,M3);	{ Read Right data }

	{ Transfer remaining data words }
	cntr = sr0;
	DO EndIn UNTIL ce;
	ax0 = dm(dmastat);
	DO Full4 UNTIL ne;
	ar  = ax0 and ay0;
Full4:	ax0 = dm(dmastat);
	ax0 = dm(dmadata);	{ Read from DMA	}
	dm(I3,M3) = ax1;	{ Write stored left sample to buffer }

	ax0 = dm(dmastat);
	DO Full5 UNTIL ne;
	ar  = ax0 and ay0;
Full5:	ax0 = dm(dmastat);
	ax0 = dm(dmadata);	{ Read from DMA	}
EndIn:	dm(I3,M3) = ay1;	{ Write stored right sample to buffer }

Finish:	{ Update status to done }
	ax0 = dm(StatBits);
	ay0 = FLG_DONE;
	ar  = ax0 OR ay0;
	dm(StatBits) = ar;
	dm(XferCtrl) = ar;

	{ Wait for enable to go low }
	ay0 = ENMASK;
	ax0 = dm(dmastat);
	DO Disabl UNTIL eq;
	ar  = dm(FieldSwitch);
	ar  = PASS ar;
	if ne CALL NewField;
	ar  = ax0 and ay0;
Disabl:	ax0 = dm(dmastat);

	{ Update status to idle }
	ax0 = dm(StatBits);
	ay0 = FLG_BUSY+FLG_DONE;
	ar  = ax0 XOR ay0;
	dm(StatBits) = ar;
	dm(XferCtrl) = ar;

	jump DataLoop;

{
 NewField - Do housekeeping needed when field IRQ has occurred
}

NewField:

	{ Copy CODEC's sticky OVR flag into StatBits, so Flyer can see it }
	{ Set for an entire field if previous field overranged }

	ax1 = dm(SERIN+3);
	ay1 = B#0010000000000000;
	ar  = ax1 AND ay1;
	ar  = FLG_CLIP;
	if eq ar = PASS 0;
	ax1 = dm(StatBits);
	ay1 = ~FLG_CLIP;
	af  = ax1 AND ay1;	{ Remove old flag }
	ar  = ar OR af;		{ Possibly set again }
	dm(StatBits) = ar;
	dm(XferCtrl) = ar;	{ Write Dma status reg }

	ax1 = 3;		{ Hold reset for next 3 samples }
	dm(ZonkOVR) = ax1;

	ar = DM(NewParam);
	ar = PASS ar;
	if eq jump NoNewArg;

	{ Update all channel parameters }
	cntr = NUMCHANS * NUMARGS;
	I5 = ^ChanArgs;
	I6 = ^ChanArgsSync;
	DO CopyArgs UNTIL ce;
	ax1 = pm(I5,M5);
CopyArgs: pm(I6,M6) = ax1;

	{ Update new CODEC controls from global parameters }
	ay1 = B#0011111100111111;
	ax1 = dm(RegArray+GLOB_OutAtten);
	ar  = ax1 AND ay1;
	ay1 = DATAWD0;				{ Start with default word }
	ar  = ar OR ay1;			{ Merge in user bits }
	dm(SEROUT+2) = ar;			{ Use from now on } 

	ay1 = B#0000111100001111;		{ OVR clear on 1st write }
	ax1 = dm(RegArray+GLOB_InputGain);
	ar  = ax1 AND ay1;
	ay1 = DATAWD1;				{ Start with default word }
	ar  = ar OR ay1;			{ Merge in user bits }
	dm(SEROUT+3) = ar;			{ Use from now on } 

NoNewArg:
	ax1 = 0;
	DM(FieldSwitch) = ax1;		{ Housekeeping done }
	DM(NewParam) = ax1;

	rts;


{............................................................................}
{............................................................................}
{ SPORT1 Tx interrupt routine						     }
{ -------------------------------------------------------------------------- }
{ This routine initializes the AD1849 control mode and then waits to see     }
{ if the codec is ready to be switched to data mode. The routine also	     }
{ initializes the transmit autobuffer with the appropriate data-mode	     }
{ control words. See the AD1849 data sheet for a complete explanation	     }
{ of control word bits.							     }
{									     }
{ Note: AY0 contains a bit mask and must NOT be modified elsewhere	     }
{............................................................................}
{............................................................................}

SETUPCONTROL:
	AX0=DM(FirstFlag);	{ First time through? }
	AF=PASS AX0;
	IF NE JUMP DECR_FIRST;	{ If so, wait until next word transmitted }
	AX0=DM(DCBflag); 
	AR=PASS AX0;
	IF EQ JUMP DCBFLG_SET;
	AX0=DM(CTRLIN);		{DCBflag has not been set yet}
	AR=AX0 XOR AY0;		{Check all incoming bits including DCB bit}
	IF EQ JUMP SET_DCB;	{Set flag if DCB was 0}
	RTI;

DCBFLG_SET:
	AX0=DM(CTRLIN);		{DCBflag was set}
	AR=AX0 AND AY0;		{Only check for DCB bit}
	IF NE JUMP SETDMODE;	{If DBC=1 ready for datamode}
	RTI;

SET_DCB:
	AX0=0;
	DM(DCBflag)=AX0;
	AX0 = CTRLWDCB;		{DCB was 0, prepare to send DCB=1}
	DM(CTRLOUT) = AX0;
	AY0 = DCBMASK;
	RTI;

DECR_FIRST:
	AX0=0;
	DM(FirstFlag)=AX0;	{If first time, set flag=0 }
	RTI;

SETDMODE:	
	IMASK=0;
	AX0=0x0418;		{Disable SPORT1}
	DM(0x3FFF)=AX0;

{ At this point we could boot page#1 and do the following data mode setup 
  after the reboot occurs.  This would free up some PM RAM for other uses.  
  
  The data mode setup initializes the transmit autobuffer control word 
  elements.  These values are not changed in this talkthrough application.
}

	I1 = ^SEROUT;
	L1=0;
	DM(I1,M1) = 0x0000;	{Reset output & input data buffers}
	DM(I1,M1) = 0x0000;	{Initialize embedded control bits}
	DM(I1,M1) = DATAWD0;	{Initialize data mode control bits}
	DM(I1,M1) = DATAWD1;
	AX0=0x001F;
	DM(0x3FF2)=AX0;		{SPORT1 control reg }
	AX0=1;
	dm(mode_sel)=AX0;	{Set D/C high }
	DM(DataMdFlg)=AX0;	{Set data mode flag high }
	RTI;

{............................................................................}
{............................................................................}
{ SPORT1 Rx interrupt routine						     }
{ -------------------------------------------------------------------------- }
{ This routine sends the incoming A/D data straight to the D/A by	     }
{ copying newly arrived data words from the SERIN buffer into the	     }
{ SEROUT buffer								     }
{............................................................................}
{............................................................................}
 
NEWDATA:
	dm(StatSave) = astat;
	ENA SEC_REG;		{ Use secondary registers }

	ax0 = dm(RecFlag);
	ar  = PASS ax0;
	if eq jump DataOut;

DataIn:
	{ While recording, loop input data to DACs }
	ax0 = dm(SERIN);	{ Get LEFT channel data from ADC } 
	dm(I2,M2) = ax0;	{ Put LEFT data into queue }
	dm(SEROUT) = ax0;	{ Output LEFT channel data to DAC }

	ax0=dm(SERIN+1); 	{ Get RIGHT channel data from ADC } 
	dm(I2,M2) = ax0;	{ Put RIGHT data into queue }
	dm(SEROUT+1) = ax0;	{ Output RIGHT channel data to ADC } 
	
	jump TestErr;

DataOut:
	{ Disregard input data while playing }

	{ Mix left channel data for all channels }

	ay1 = I2;		{ Store current location }
	M2 = TBUFLEN;		{ Length of entire channel buffer }

	{ Setup mix operation }
	cntr = NUMCHANS - 1;
	mr = 0, my0 = pm(I4,M4), mx0 = dm(I2,M2);
	DO Mix1 UNTIL ce;
Mix1:	mr = mr + mx0 * my0 (SU), my0 = pm(I4,M4), mx0 = dm(I2,M2);
	mr = mr + mx0 * my0 (SU);
	mr = mr (RND);
	if mv sat mr;
	dm(SEROUT) = mr1;	{ output LEFT channel data }

	{ Mix right channel data for all channels }

	I2 = ay1;		{ Retrieve location }
	modify (I2,M1);		{ Point to RIGHT channel data }
	ay1 = I2;		{ Save current location }

	{ Setup mix operation }
	cntr = NUMCHANS - 1;
	mr = 0, my0 = pm(I4,M4), mx0 = dm(I2,M2);
	DO Mix2 UNTIL ce;
Mix2:	mr = mr + mx0 * my0 (SU), my0 = pm(I4,M4), mx0 = dm(I2,M2);
	mr = mr + mx0 * my0 (SU);
	mr = mr (RND);
	if mv sat mr;
	dm(SEROUT+1) = mr1;	{ output RIGHT channel data } 

	{ Update pointers}
	M2 = 1;			{ Restore Modulo register }
	I2 = ay1;		{ Retrieve location }
	modify (I2,M1);		{ Point to next data value }

{............................................................................}
{ Error checking section (optional). Incoming control words are compared     }
{ to the outgoing control words.					     }
{............................................................................}

TestErr:
	ar = dm(ZonkOVR);
	af = PASS ar;
	if eq jump skipit;
	ar = af-1;
	dm(ZonkOVR) = ar;
	if ne jump skipit;

	{ Release OVR bit to CODEC so it can set OVR on input overrange }
	ar = dm(SEROUT+3);
	ay0 = B#0010000000000000;
	ar = ar OR ay0;
	dm(SEROUT+3) = ar;
skipit:

	jump RxEnd;

	ax0= dm(SEROUT+2);	{ Read known output control word }
	ay0= dm(SERIN+2);	{ Read newly received control word }
	ar = ax0-ay0;		{ Compare, they should be the same }
	if eq jump no_error;	{ If same, no error }
	ax0= dm(SyncFlag);	{ If not, read SyncFlag }
	ar = PASS ax0;		{ Test for 0 or 1 }
	if ne jump reset_sport;	{ If 1, second failure, reset SPORT1 }

	ar = PASS 1;		{ Else, first failure, set SyncFlag }
	dm(SyncFlag)=ar;
	jump RxEnd;

reset_sport:
	si=0x418;		{ Disable SPORT1 }
	dm(0x3fff)=si;		
	si=0xc18;		{ Re-enable SPORT1 }
	dm(0x3fff)=si;

no_error:
	ar = PASS 0;
	dm(SyncFlag)=ar;	{ reset SyncFlag }

RxEnd:
	DIS SEC_REG;
	astat = dm(StatSave);
	rti;			{ return }

{*
 * - Vertical Interrupt service routine
 *}

VertInt:
	dm(StatSave) = astat;
	ENA SEC_REG;

	ax0 = 1;
	DM(FieldSwitch) = ax0;		{ Do more housekeeping in main code }

	{ If recording, update headers }
	ax0 = dm(RecFlag);
	ar  = PASS ax0;
	if eq jump NxtFld;

	ax0 = I2;
	ay0 = dm(CodecBuff);
	I2  = ay0;		{ Reset pointer to start of buffer }
	ar  = ax0 - ay0;	{ Compute amount of data collected }
	dm(I2,M2) = ar;		{ Make cute little header thingy }
	dm(I2,M2) = 0x0000;
	
NxtFld:	{ Toggle field bit }
	ax0 = dm(Field);
	ay0 = 0x0001;
	ar  = ax0 xor ay0;
	dm(Field)  = ar;

	{ Calculate buffer start offset }
	if ne jump fld1;
	ay0 = 0x0000;	{ Field = 0: 1st half for DMA, 2nd half for CODEC }
	ay1 = BUFLEN;
	jump NewPtrs;

Fld1:	ay0 = BUFLEN;	{ Field = 1: 2nd half for DMA, 1st half for CODEC }
	ay1 = 0x0000;

NewPtrs:
	{ Update buffer pointers }
	ax0 = CH0BASE;
	ar  = ax0 + ay0;
	dm(ChanPtrs + 0) = ar;

	ax0 = CH1BASE;
	ar  = ax0 + ay0;
	dm(ChanPtrs + 1) = ar;

	ax0 = CH2BASE;
	ar  = ax0 + ay0;
	dm(ChanPtrs + 2) = ar;

	ax0 = CH3BASE;
	ar  = ax0 + ay0;
	dm(ChanPtrs + 3) = ar;
	
	{ Serial points to other buffer }
	ax0 = CH0BASE;
	ar  = ax0 + ay1;
	I2  = ar;
	dm(CodecBuff) = ar;

	{ Skip header on serial buffer }
	modify (I2,M2);
	modify (I2,M2);

	{ Update direction flag }
	ax0 = dm(RegArray+GLOB_Flags);
	ay0 = GFLG_DIR;
	ar  = ax0 and ay0;
	dm(RecFlag) = ar;

	DIS SEC_REG;
	astat = dm(StatSave);
	rti;

.ENDMOD;
