{****************************************************************************
ADSP-2105 - AD1849(SOUNDPORT)  INTERFACE PROGRAM
Oli Josefsson, Kapriel Karagozyan, & Noam Levine     Analog Devices Jan, 1992

This program is written to run on an ADSP-2105.

For the hardware connections between the ADSP-2105 and AD1849 please refer
to the diagrams shown on the AD1849 data sheet.

Note: RFS1 and TFS1 MUST be tied together in order to initialize and 
operate the codec.

*****************************************************************************

In its current condition, this program can be used "as is" to perform straight
talkthrough (at 44.1 KHz sample rate) on both left and right channels of the 
AD1849. The incoming audio data from the 16bit ADCs is placed in a short buffer 
and immediately sent out to the 16bit DACs.

The initial setup and handshaking with the AD1849 starts with the "START"
routine. A state machine to perform the handshaking is executed in software.
Once the AD1849 is configured properly, the processor enters the WAIT_DATA
loop and waits for serial port interrupt requests.

This program is booted into the ADSP-2105 on power-up.

*****************************************************************************

Modified for use on DHD by Charles Steinkuehler on 10-9-92
Modified for DHD2A by Charles Steinkueler on 12-27-93
Modified for DHD3 by Charles Steinkuehler on 3-12-94

  Single direction DMA channel with direction flag port
  Updated to current ADI 1849 interface code
   -equal length transmit and recieve circular buffers
   -direct addressing of transmit buffer
   -this frees one index register (I3) and prevents problems due to clk rate

*****************************************************************************}

.MODULE/ABS=0/BOOT=0  AD1849;

.CONST CHANMASK	= B#00000001;	      { 2 CHANNELS }
.CONST LENMASK	= B#01110000;	      { 7 BLKS MAX }
.CONST ENMASK	= B#00000001; 	      { 1 = ENABLED, 0 = DISABLED }
.CONST DIRMASK	= B#00000010; 	      { 1 = AUD TO DMA, 0 = DMA TO AUD }
.CONST FULLMASK	= B#00010000; 	      { 1 = DMA REG FULL, 0 = DMA REG EMPTY }
.CONST CTRLWD0	= B#0010000000010100; {DCB=0,AC=1,DFR=2,ST=1,DF=00}
.CONST CTRLWDCB	= B#0010010000010100; {DCB=1,AC=1,DFR=2,ST=1,DF=00}
.CONST CTRLWD1	= B#0001001000000000; {MCK=1,FSEL=0,MS=1,TXDIS=0,ENL=0,ADL=0}
.CONST CTRLWD2	= B#0000000000000000; {PIO = 00}
.CONST CTRLWD3	= B#0000000000000000; {REVID=NC}
.CONST DCBMASK	= B#0000010000000000; {DCB MASK}

.CONST BUFLEN	= 0x0800; 	      { 4 DMA blocks - 1024 Words }
.CONST CH0BASE  = 0x0000;
.CONST CH1BASE  = 0x0800;
.CONST CH2BASE  = 0x1000;
.CONST CH3BASE  = 0x1800;


.VAR/CIRC  	CTRLIN[4];      { circular buffers for data input and }
.VAR/CIRC  	CTRLOUT[4];     { output for data mode and control mode }
.VAR       	FIRST_FLG;	
.VAR       	DCB_FLG;
.VAR       	DMODE_FLG;
.VAR       	sync_flag;


.VAR/CIRC  	SERIN[4];	{ SERIAL INPUT AUTOBUFFER }
.VAR/CIRC  	SEROUT[4];	{ SERIAL OUTPUT AUTOBUFFER }

.VAR       	dir;		{ Direction bit 1=Rec,0=Play }
.VAR       	fld;		{ Field bit (toggles) }
.VAR       	stat;		{ DMA status 0=idle,1=busy,3=done }
.VAR       	len;		{ Length }
.VAR       	serptr;		{ Ptr to start of serial buffer }
.VAR		lastdata;	{ Last valid data value on play }
.VAR       	chptr[4];	{ Ptr to start of DMA bufs, 1 per chan }

.port   	mode_sel;	{ I_O reg for a/d control/data mode pin }
.port   	dmastat;	{ Dma status port }
.port   	dmadata;	{ Dma data port }
.port		Reg1;		{ R=BlkLen/Channel reg/W=Dma Status }
.port		Reg2;		{ R=Direction reg/W=Not Used }
.port		Cpu3;		{ Currently not used }


	{ Vector Table }

	JUMP START;nop;nop;nop;         {restart interrupt}
	JUMP VertInt;nop;nop;nop;       {IRQ2 int, vertical rate}
	RTI;nop;nop;nop;                {SPORT0 tx not used}
	RTI;nop;nop;nop;                {SPORT0 rx not used}
	JUMP SETUPCONTROL;nop;nop;nop;  {SPORT1 transmit int}
	JUMP NEWDATA;nop;nop;nop;       {SPORT1 receive int}
	RTI;nop;nop;nop;                {timer int,not used}

START:  AX0=0; 
        dm(mode_sel)=AX0;    {set the AD1849 D/C pin low}
        M0=1;
        L0=%CTRLIN;
        I0=^CTRLIN;
        M1=1;
        L1=%CTRLOUT;
        I1=^CTRLOUT;
        AX0=1;
        DM(FIRST_FLG)=AX0;	
        DM(DCB_FLG)=AX0;	
        AX0=0;
        DM(DMODE_FLG)=AX0;      { in control mode }

{Initialize contol mode output buffer}
	DM(I1,M1)= CTRLWD0;
	DM(I1,M1)= CTRLWD1;
	DM(I1,M1)= CTRLWD2;
	DM(I1,M1)= CTRLWD3;
	L2 = 0;                  { linear addressing for register }
	I2 = 0x3fef;             { point to last DM cntrl reg }

{3FEF}	DM(I2,M1) = 0x0283;      { sport1 autobuffer register }
{3FF0}	DM(I2,M1) = 383;         { rfsdiv1	}
{3FF1}	DM(I2,M1) = 849;         { sclkdiv1 	}
{3FF2}	DM(I2,M1) = B#0100000100011111;  { sport1 control register:                                                      
                                           internal sclk & rfs                                                             
                                           normal framing mode                                                             
                                           frame sync not inverted
                                           16-bit word length }

	I2=0x3ffb;

{3FFB}	DM(I2,M1) = 0x0000;      { TSCALE register }
{3FFC}	DM(I2,M1) = 0x0000;      { TCOUNT register }
{3FFD}	DM(I2,M1) = 0x0000;      { TPERIOD register,initializing value
                                   for TCOUNT after every interrupt }

{3FFE}	DM(I2,M1) = 0x0000;      { external data memory waits=0 }


	ICNTL=0x00 ;
	IMASK=B#000100;          { only SPORT1 tx interurpt enabled
                                   initially while in control mode }
 
{.....Set bit test mask for DCB bit, used in tx interrupt state machine....}

	AY0 = CTRLWD0; { test mask for DCB bit } 

{.....       send first control word to switch codec to data mode       .....}
	AX0=DM(I1,M1);       { send first 16bits of ctrl word }                         
	TX1=AX0;

{3FFF}	DM(I2,M1) = 0x0c18;  { system control reg: sport1 enabled }

{..... Wait for an interrupt indicating that transmit register is ready for 
       new data and that the 2105 has received a 16bit word             .....}

WAIT1:	AX1=DM(DMODE_FLG);   { check dmode flag }
	AR=PASS AX1;
	IF GT JUMP GO_DMODE; { if set, in data mode }
	JUMP WAIT1;          { else, wait for initialization to be completed                                              
                               from tx interrupt routine }

GO_DMODE:  
	L0=%SERIN;          { init I0, L0 for rx autobuffer }
	I0=^SERIN;
	M0=1;
	L1=%SEROUT;         { init I1, L1 for tx autobuffer }
	I1=^SEROUT;
	M1=1;

	{ DAG 2: Serial queue }
	I2 = CH0BASE;
	L2 = BUFLEN;
	M2 = 1;

	{ DAG 3: DMA queue }
	I3 = CH0BASE + BUFLEN;
	L3 = BUFLEN;
	M3 = 1;

	{ DAG 4: Coefficent storage }

	{ DAG 7: Indirect access }
	I7 = 0;
	L7 = 0;
	M7 = 0;

	{ Initalize variables }
	ar = pass 0;
	dm(fld) = ar;
	dm(stat)= ar;
	ax0 = 1;
	dm(dir) = ax0;

	{ Initalize Channel pointers }
	ax0 = CH0BASE + BUFLEN;
	dm(chptr + 0) = ax0; 
	ax0 = CH1BASE + BUFLEN;
	dm(chptr + 1) = ax0; 
	ax0 = CH2BASE + BUFLEN;
	dm(chptr + 2) = ax0; 
	ax0 = CH3BASE + BUFLEN;
	dm(chptr + 3) = ax0; 


	AX0=DM(I1,M1);		{ send first 16bits of data }
	TX1=AX0;
	AX0=0X0c18;
	DM(0X3FFF)=AX0;		{ turn on sport1 }

	IFC=B#000000111111;	{ clear all pending interrupts }
	nop;			{ cycle for IFC latency }
	ICNTL=B#00100;		{ IRQ2 edge sensitive, int nesting off }
	IMASK=B#100010;		{ IRQ2 & sport1 rx interrupt on }
	{IMASK=B#000000;}		{ all interrupts off }

DataLoop: 
	{ Check for new enable }
	ax0 = dm(dmastat);
	ay0 = ENMASK;
	ar  = ax0 and ay0;
	if eq jump DataLoop;
StartXfer:
	{ Update status to busy }
	ax0 = 1;		
	dm(Reg1) = ax0;		{ Write Dma status reg }
	dm(stat) = ax0;

	{ Get channel and blocklen }
	ax0 = dm(Reg1);		{ Read block length/channel reg }

	{ Calculate buffer pointer }
	ay0 = CHANMASK;
	ar  = ax0 and ay0;	{ Mask for channel bits }
	ay0 = ^chptr;		{ Pointer to buffer address array }
	ar  = ar + ay0;		{ Offset for channel }
	I7  = ar;		{ Use as an address }
	ax1 = dm(I7,M7);	{ Read in buffer start address }
	I3  = ax1;		{ Use buffer address for DMA DAG }
	
	{ Calculate buffer length }
	ay0 = LENMASK;
	ar  = ax0 and ay0;	{ Mask for length bits }
	sr  = lshift ar by 4 (LO);	{ Shift to calculate word count }
	ax1 = sr0;		{ Result to ax1 }
	dm(len) = sr0;

	{ Check for zero length }
	ar  = pass ax1;
	if eq jump finish;	

	{ Branch to In or Out }
	ay0 = FULLMASK;		{ Setup for quick test }
	ay1 = dm(dir);		
	ar  = pass ay1;
	{if eq jump DmaIn;}

DmaOut:
	cntr = ax1;
	DO EndOut UNTIL ce;
	DO Empty Until eq;
	ax0 = dm(dmastat);	{ Read dma hardware status register }
Empty:	ar  = ax0 and ay0;	{ And with Fullmask }
	nop;
	nop;
	nop;
	nop;
	nop;
	si = cntr;
	sr = LSHIFT si BY -5 (HI);
	dm(Reg2) = sr1;	{ Debug }
	sr = LSHIFT sr0 BY -8 (LO);
	ay0 = sr0;
	ar = ay0 + 1;
	dm(Reg1) = ar;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	nop;
	ax0 = dm(I3,M3);	{ Read next word to transfer }
EndOut:	dm(dmadata) = ax0;	{ Write to DMA }

	jump Finish;



DmaIn:
	{ Read first word (length) }
	DO Full1 UNTIL ne;	{ Loop until full flag true }
	ax0 = dm(dmastat);	{ Read dma hardware status register }
Full1:	ar  = ax0 and ay0;	{ And with Fullmask }
	ay1 = dm(dmadata);	{ Read from DMA }
	dm(I3,M3) = ay1;	{ Write to Buffer }
	{ Read second word (Pad) }
	DO Full2 UNTIL ne;	{ Loop until full flag true }
	ax0 = dm(dmastat);	{ Read dma hardware status register }
Full2:	ar  = ax0 and ay0;	{ And with Fullmask }
	ax0 = dm(dmadata);	{ Read from DMA }
	dm(I3,M3) = ax0;	{ Write to Buffer }

	{ Subtract 2 from length }
	ax0 = 2;
	ar  = ay1 - ax0;
	ay1 = ar;

	{ Read length words }
	cntr = ay1;		{ Load counter with word count }
	DO EndIn1 UNTIL ce;	{ Loop until counter expires }
	DO Full3 UNTIL ne;	{ Loop until full flag true }
	ax0 = dm(dmastat);	{ Read dma hardware status register }
Full3:	ar  = ax0 and ay0;	{ And with Fullmask }
	ax0 = dm(dmadata);	{ Read from DMA }
EndIn1:	dm(I3,M3) = ax0;	{ Write to Buffer }

	{ Calculate remaining data }
	ar  = ax1 - ay1;

	{ Transfer remaining data }
	cntr = ar;
	ax1  = ax0;		{ Save last valid value }
	DO EndIn2 UNTIL ce;	{ Loop until counter expires }
	DO Full4 UNTIL ne;	{ Loop until full flag true }
	ax0 = dm(dmastat);	{ Read dma hardware status register }
Full4:	ar  = ax0 and ay0;	{ And with Fullmask }
	ax0 = dm(dmadata);	{ Read from DMA }
EndIn2:	dm(I3,M3) = ax1;	{ Write last valid data to Buffer }

Finish:	{ Update status to done }
	ax0 = 3;		
	dm(Reg1) = ax0;		{ Write Dma status reg }
	dm(stat) = ax0;

	{ Wait for enable to go low }
	ay0 = ENMASK;		
	DO Enable UNTIL eq;
	ax0 = dm(dmastat);
Enable: ar  = ax0 and ay0;

	{ Update status to idle }
	ax0 = 0;		
	dm(Reg1) = ax0;		{ Write Dma status reg }
	dm(stat) = ax0;

	jump DataLoop;

{............................................................................}
{............................................................................}
{ sport1 tx interrupt routine	                                             }
{ -------------------------------------------------------------------------- }
{ This routine initializes the AD1849 control mode and then waits to see     }
{ if the codec is ready to be switched to data mode. The routine also        }
{ initializes the transmit autobuffer with the appropriate data-mode         }
{ control words. See the AD1849 data sheet for a complete explanation        }
{ of control word bits.                                                      }
{	                                                                     }
{ Note: AY0 contains a bit mask and must NOT be modified elsewhere           }
{............................................................................}
{............................................................................}

SETUPCONTROL:    AX0=DM(FIRST_FLG);   { first time through ? }
         AF=PASS AX0;
         IF NE JUMP DECR_FIRST;  { if so, wait until next word transmitted}
         AX0=DM(DCB_FLG); 
         AR=PASS AX0;
         IF EQ JUMP DCBFLG_SET;
         AX0=DM(CTRLIN);         {DCB_FLG has not been set yet}
         AR=AX0 XOR AY0;         {check all incoming bits including DCB bit}
         IF EQ JUMP SET_DCB;     {set flag if DCB was 0}
         RTI;

DCBFLG_SET:  AX0=DM(CTRLIN);     {DCB_FLG was set}
         AR=AX0 AND AY0;         {only check for DCB bit}
         IF NE JUMP SETDMODE;    {if DBC=1 ready for datamode}
         RTI;

SET_DCB: AX0=0;
         DM(DCB_FLG)=AX0;
         AX0 = CTRLWDCB; {DCB was 0, prepare to send DCB=1}
         DM(CTRLOUT) = AX0;
         AY0 = DCBMASK;
         RTI;

DECR_FIRST:  AX0=0;
         DM(FIRST_FLG)=AX0;     { if first time, set flag=0 }
         RTI;

SETDMODE:        
	IMASK=0;
	AX0=0X0418;              {disable sport1}
	DM(0X3FFF)=AX0;

{ At this point we could boot page#1 and do the following data mode setup 
  after the reboot occurs.  This would free up some PM RAM for other uses.  
  
  The data mode setup initializes the transmit autobuffer control word elements. 
These values are not changed in this talkthrough application.    }
                 
	I1 = ^SEROUT;
	L1=0;
	DM(I1,M1) = 0x0000;       {reset output & input data buffers}
	DM(I1,M1) = 0x0000;       {initialize embedded control bits}
	DM(I1,M1) = B#1100000000000000; {OM1=1,OM1=1,LO=0,SM=0,RO=0}
	DM(I1,M1) = B#0000000011110000; {PIO=00,OVR=0,IS=0,LG=0,MA=15,RG=0}
	AX0=0X001F;
	DM(0X3FF2)=AX0;           { sport1 control reg }              
	AX0=1;
	dm(mode_sel)=AX0;         { set D/C high }
	DM(DMODE_FLG)=AX0;        { set data mode flag high }
	RTI;

{............................................................................}
{............................................................................}
{ sport1 rx interrupt routine                                                }
{ -------------------------------------------------------------------------- }
{ This routine sends the incoming a/d data straight to the d/a by            }
{ copying newly arrived data words from the SERIN buffer into the           }
{ SEROUT buffer	                                                           }
{............................................................................}
{............................................................................}
 
NEWDATA:  
	ENA SEC_REG;		{ USE SECONDARY REGS }

	ax0 = dm(dir);
	ar  = pass ax0;
	if eq jump DataOut;

DataIn:
	{ Loop through input data while recording }
	ax0 = dm(SERIN);        { get LEFT channel data } 
        dm(I2,M2)    = ax0;	{ put LEFT data into queue }
        dm(SEROUT)   = ax0;    	{ output LEFT channel data }

        ax0=dm(SERIN+1); 	{ get RIGHT channel data } 
        dm(I2,M2)    = ax0;	{ put RIGHT data into queue }
        dm(SEROUT+1) = ax0;    	{ output RIGHT channel data } 
	
	jump TestErr;

DataOut:
	{ Discard input data while playing }
	ax0 = dm(I2,M2);        { get LEFT channel data } 
        dm(SEROUT)   = ax0;    	{ output LEFT channel data }

        ax0 = dm(I2,M2); 	{ get RIGHT channel data } 
        dm(SEROUT+1) = ax0;    	{ output RIGHT channel data } 
	
{............................................................................}
{ Error checking section (optional). Incoming control words are compared     }
{ to the outgoing control words.                                             }
{............................................................................}

TestErr:
	
        ax0=dm(SEROUT+2);  { read known output control word }
        ay0=dm(SERIN+2);   { read newly received control word }
        ar=ax0-ay0;         { compare, they should be the same }
        if eq jump no_error;{ if same, no error }
        ax0=dm(sync_flag);  { if not, read SYNC_FLAG }
        ar=pass ax0;        { test for 0 or 1 }
        if ne jump reset_sport;{ if 1, second failure, reset SPORT1 }

        ar=pass 1;          { else, first failure, set SYNC_FLAG }
        dm(sync_flag)=ar;	
	DIS SEC_REG;
        rti;                { return }

reset_sport:
        si=0x418;           { disable SPORT1 }
        dm(0x3fff)=si;		
        si=0xc18;           { re-enable SPORT1 }
        dm(0x3fff)=si;

no_error: ar=pass 0;
        dm(sync_flag)=ar;   { reset SYNC_FLAG }
	DIS SEC_REG;
        rti;                { return }

{ -------------------------------------------------------------------------- }
{ The SEROUT buffer is twice as long as the SERIN buffer, since it           }
{ contains control words, as well as output data. This program ignores the   }
{ control words arriving back from the AD1849 during the data mode, except   }
{ for the purpose of error checking.                                         }
{ -------------------------------------------------------------------------- }

{*
 * - Vertical Interrupt service routine
 *}
               
VertInt:
	ena SEC_REG;

	{ update headers for record }
	ax0 = dm(dir);
	ar  = pass ax0;
	if eq jump NxtFld;
	ax0 = I2;
	ay0 = dm(serptr);
	I2  = ay0;
	ar  = ax0 - ay0;
	dm(I2,M2) = ar;
	dm(I2,M2) = 0x0000;
	

	{ Toggle field bit }
NxtFld:
	ax0 = dm(fld);
	ay0 = 0x0001;
	ar  = ax0 xor ay0;
	dm(fld)  = ar;
	{ Calculate buffer start offset }
	if ne jump fld1;
	ay0 = 0x0000;
	ay1 = BUFLEN;
	jump NewPtrs;
Fld1:
	ay0 = BUFLEN;
	ay1 = 0x0000;

	{ Update buffer pointers }
NewPtrs:
	ax0 = CH0BASE;
	ar  = ax0 + ay0;
	dm(chptr + 0) = ar;

	ax0 = CH1BASE;
	ar  = ax0 + ay0;
	dm(chptr + 1) = ar;

	ax0 = CH2BASE;
	ar  = ax0 + ay0;
	dm(chptr + 2) = ar;

	ax0 = CH3BASE;
	ar  = ax0 + ay0;
	dm(chptr + 3) = ar;
	
	{ Serial points to other buffer }
	ax0 = CH0BASE;
	ar  = ax0 + ay1;
	dm(serptr) = ar;

	dm(serptr) = I2;	{ Store serial buffer base }

	{ Skip header on serial buffer }
	ax0 = dm(I2,M2);
	ax0 = dm(I2,M2);

	{ Update direction flag }
	ax0 = dm(Reg2);
	ay0 = 0x0001;
	ar  = ax0 and ay0;
	dm(dir) = ar;

	{ Check channel enables and set gain parameters }

	dis SEC_REG;
	rti;

.ENDMOD;
