/****************************************************************************
**
**                              mem_lib
**
**  A library to manage the storage and freeing of memory blocks.
** The library creates and manages a liked list of memory arrays
** and provides a single function call to free all memory blocks
** stored in the array;
**
**
**
**
****************************************************************************/

#include <stdio.h>
#include <stdlib.h>

#include "bb_common.h"
#include "mem_lib.h"
#include "err_lib.h"


/* Initialize list */


/****************************************************************************
**
**   struct mem_list  *initialize_mem_list( int num_pointers_per_block)
**
**
** DESC:
**   Allocates a mem_list structure.  Returns a pointer to the mem list
** header.  Returns NULL on error.
**
**
** INPUTS:
**  int    num_pointers_per_block     - # of poiners in each mem_block
**
** OUTPUTS:
**
** RETURNS:
**  struct mem_list  *                - ptr to mem_list header
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
struct mem_list  *initialize_mem_list( int num_ptrs_per_block)
{
    struct mem_list  *mem_list_ptr;

    /* attempt to allocate a header block */

    if( (mem_list_ptr = calloc(1, sizeof(struct mem_list))) == NULL)
    {
        return(NULL);
    }

    /* initialize the structure */
    mem_list_ptr->status               = INITIALIZED;
    mem_list_ptr->total_elements       = 0;
    mem_list_ptr->num_used             = 0;
    mem_list_ptr->num_blocks           = 0;
    mem_list_ptr->num_ptrs_per_block   = num_ptrs_per_block;
    mem_list_ptr->block_head           = (struct mem_block *)NULL;
    mem_list_ptr->block_tail           = (struct mem_block *)NULL;

    return(mem_list_ptr);
}




/* Remove List */


/****************************************************************************
**
** int remove_mem_list( struct mem_list *head)
**
** DESC:
**  remove_mem_list() frees all the pointers in the mem list and removes
** the pointer blocks themselves.  The memory list is completely freed
** and must be re-initiailzied if it is to be used again.
**
**
** INPUTS:
**  struct mem_list *head  -- header block for mem_list
**
** OUTPUTS:
**
** RETURNS:
**   OK      -- success
**   ERROR   -- list not initialized
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int remove_mem_list( struct mem_list *head)
{
    struct mem_block    *block_temp, *temp;
    int                 quitflag, i;

    /* make sure the list is initialized  */
    if( (head->status != INITIALIZED) )
    {
        log_message(ERROR, "mem list not init in remove_mem_list_ptr()");
        return(ERROR);
    }

    /* sequentially step through the list, freeing the memory */

    if (head->num_used > 0)
    {
        block_temp       = head->block_head;
        quitflag         = FALSE;

        while (!quitflag)
        {

            for(i=0; i<head->num_ptrs_per_block; i++)
            {
                if (block_temp->values[i] != NULL)
                {

                    free(block_temp->values[i]);
                    head->num_used--;
                }
            }

            /* free the array */
            free (block_temp->values);

            if (block_temp->next != NULL)
            {
                temp       = block_temp;
                block_temp = block_temp->next;
                free(temp);
            }
            else
            {
               quitflag = TRUE;
            }

        }
    }

    /* set the header block's initialized flag to UNINITIALIZED */
    /* so that, if the user attempts to call a list function after */
    /* the block is freed using the old pointer, it will error out */
    head->status = UNINITIALIZED;

    /* free the header block */
    free(head);

    return(OK);
}



/* Insert pointer */



/****************************************************************************
**
**  int  insert_mem_list_ptr( struct mem_list *head, void *pointer)
**
** DESC:
**  insert_mem_list_ptr() adds an element to the memory list.  If the
** memory list is currently full (or no entries have been allocated),
** a new block is allocated.
**
**  The function searches all the entries until it finds an empty
** slot (NULL).  This will get slower as more elements are added, but
** since entries should be added only infrequently, this shouldn't be
** a problem.
**
**
** INPUTS:
**   struct mem_list    *head     -- pointer to mem list header
**   void               *pointer  -- memory pointer to be added
**
** OUTPUTS: none
**
** RETURNS:
**   OK      -- element added
**   ERROR   -- problem allocating element
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  insert_mem_list_ptr( struct mem_list *head, void *pointer)
{

    struct mem_block    *block_temp;
    void                **ptr_temp;

    /* make sure the list is initialized */
    if (head->status != INITIALIZED)
    {
        log_message(ERROR, "mem list not init in insert_mem_list_ptr()");
        return(ERROR);
    }

    /* if the list is full (or completely empty), allocate a block for ptrs */

    if( head->num_blocks == 0 )
    {
        if( (head->block_head = calloc(1, sizeof(struct mem_block))) == NULL)
        {
            return(ERROR);
        }

        if( (head->block_head->values = calloc(head->num_ptrs_per_block,
                                        sizeof(void *))) == NULL)
        {
            free(head->block_head);
            return(ERROR);
        }

        /* fill out information */
        head->num_blocks++;
        head->block_tail           = head->block_head;
        head->total_elements      += head->num_ptrs_per_block;

        head->block_head->num_used = 0;
        head->block_head->next     = NULL;

    }
    else if( head->total_elements == head->num_used ) //full, add new block
    {

        if( (block_temp = calloc(1, sizeof(struct mem_block))) == NULL)
        {
            return(ERROR);
        }

        if( (block_temp->values = calloc(head->num_ptrs_per_block,
                                        sizeof(void *))) == NULL)
        {
            free(block_temp);
            return(ERROR);
        }

        /* fill out information */
        head->num_blocks++;
        head->block_tail->next     = block_temp;  // add to linked list
        head->block_tail           = block_temp;  // now make tail element
        head->total_elements      += head->num_ptrs_per_block;
        head->block_tail->next     = NULL;
    }

    /* we are now garunteed a block with an empty cell */

    /* find an empty cell, search until an empty element (= 0) is found */
    /* before an element by element search, make sure the block isn't already*/
    /* full to save time */

    block_temp = head->block_head;

    /* find a block that is not full */

    while (block_temp->num_used == head->num_ptrs_per_block)
    {
        block_temp = block_temp->next;
    }

    /* there is at least one empty cell in the current block, find it */

    ptr_temp = block_temp->values;

    while (*ptr_temp != (void *)NULL);
    {
        ptr_temp++;
    }

    /* save the value and increment the counters */
    *ptr_temp = pointer;
    head->num_used++;
    block_temp->num_used++;

    return(OK);
}




/* Remove pointer */


/****************************************************************************
**
** int  free_mem_list_ptr( struct mem_list *head, void *pointer, int free_flag)
**
**
** DESC:
**  free_mem_list_ptr removes a pointer from the resource list.  It does
** a simple, sequential search through the list until it finds the pointer
** and then zeros out that entry and decrements the resource counters.
**
**  if free_flag is TRUE, the memory pointed to by the block is also
** freed.
**
**
**
** INPUTS:
**   struct mem_list    *head     -- pointer to mem list header
**   void               *pointer  -- memory pointer to be added
**   int                free_flag -- TRUE = free() block, FALSE = don't free
**
** OUTPUTS: none
**
** RETURNS:
**   OK      -- element added
**   ERROR   -- element not found or list not initialized
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  free_mem_list_ptr( struct mem_list *head, void *pointer, int free_flag)
{

    struct mem_block    *block_temp;
    int                 quitflag, i;

    /* make sure the list is initialized and not empty */
    if( (head->status != INITIALIZED) || (head->num_used == 0) )
    {
        log_message(ERROR, "mem list not init in free_mem_list_ptr()");
        return(ERROR);
    }

    /* sequentially search through the list for the pointer */
    block_temp       = head->block_head;
    quitflag         = FALSE;

    while (!quitflag)
    {
        /* scan the block for the pointer value */

        for(i=0; i<head->num_ptrs_per_block; i++)
        {
            if (block_temp->values[i] == pointer)
            {
                /* zero out value and decrement counters */
                if (free_flag == TRUE)
                {
                    free(pointer);
                }
                block_temp->values[i]  = 0;
                block_temp->num_used--;
                head->num_used--;

                return(OK);
            }
        }

        /* end of block, go to next block, quit if end of list */
        if (block_temp->next != NULL)
        {
            block_temp = block_temp->next;
        }
        else
        {
            quitflag = TRUE;
        }
    }

    /* if we got this far, the pointer was not found */
    return(ERROR);
}








/*--- Spare Template ---*/


/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/



