/****************************************************************************
**                           Project Library
**
**
**
**
**
****************************************************************************/

/*-- System Include files --*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>

#include <graphics/displayinfo.h>
#include <libraries/dos.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>



/*-- User Include Files --*/

#include "tl_common.h"
#include "tl_init.h"
#include "mem_lib.h"
#include "err_lib.h"
#include "timecode_lib.h"
#include "bb_util_lib.h"
#include "prj_lib.h"




/*-- Global variables --*/

struct prj_clip_list    g1_list;
struct prj_clip_list    v1_list;
struct prj_clip_list    a1_list;
struct prj_clip_list    a2_list;
struct prj_clip_list    a3_list;
struct prj_clip_list    a4_list;




/*-- initialize clip list --*/

/****************************************************************************
**
** int init_clip_list( struct prj_clip_list  *cliplist,
**                     struct hitbox_struct  extents);
**
** DESC:
**  initializes the clip_list structure
**
** INPUTS:
**  struct prj_clip_list        *cliplist;
**  struct hitbox_struct        extents;
**
** OUTPUTS:  None
**
** RETURNS:
**  OK  -- success
**  ERROR  -- failure (struct already initialized)
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int init_clip_list( struct prj_clip_list  *cliplist,
                    struct hitbox_struct  extents)
{

    /* don't init if already initialized */
    if (cliplist->status != UNINITIALIZED)
    {
        return(ERROR);
    }

    /* initialize values */

    cliplist->status = INITIALIZED;
    cliplist->num_elements = 0;

    cliplist->head   = NULL;
    cliplist->tail   = NULL;
    cliplist->cursor = NULL;

    cliplist->extents.top    = extents.top;
    cliplist->extents.left   = extents.left;
    cliplist->extents.bottom = extents.bottom;
    cliplist->extents.right  = extents.right;

    cliplist->disp_start  = NULL;
    cliplist->disp_end    = NULL;

    return(OK);
}


/*-- insert clip into list --*/

/****************************************************************************
**
**  int insert_clip( struct prj_clip_list    *cliplist,
**                   struct prj_clip_struct  *clip);
**
** DESC:
**  inserts a clip after the current cursor clip. If the "head" is NULL, the
** the clip is at the start.  If the cursor is NULL, the clip is inserted
** before the "head" clip.
**
**  The cursor is set to the clip inserted, to build an initial clip list,
** no changes need to be made
**
**
**
** INPUTS:
**  struct prj_clip_list    *cliplist  -- cliplist to insert into
**  struct prj_clip_struct  *clip      -- clip to insert
**
** OUTPUTS:
**
** RETURNS:
**  OK  -- success
**  ERROR  -- failure
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int insert_clip( struct prj_clip_list    *cliplist,
                 struct prj_clip_struct  *clip)

{
    if (cliplist->status != INITIALIZED)
    {
        return(ERROR);
    }

    if (cliplist->head == NULL)      // first element
    {
        cliplist->head = clip;
        clip->next = NULL;
        clip->prev = NULL;
    }
    else if (cliplist->cursor == NULL)  // append before first element
    {
        clip->next = cliplist->head;
        clip->next->prev = clip;
        clip->prev = NULL;
        cliplist->head = clip;
    }
    else                                // just insert, after cursor
    {
        clip->next = cliplist->cursor->next;
        clip->prev = cliplist->cursor;
        clip->prev->next = clip;
    }
    cliplist->cursor = clip;
    cliplist->num_elements++;

    /* change tail if necessary */
    if (clip->next == NULL)
    {
        cliplist->tail = clip;
    }

    return(OK);
}



/*-- remove clip from list --*/

/****************************************************************************
**
** struct prj_clip_struct *remove_clip( struct prj_clip_list    *cliplist)
**
** DESC:
**  removes the clip at the current "cursor" position in the cliplist.
** and returns that pointer to the calling program.
**
**  The cursor is set to the clip before the deleted one.  It automatically
** checks for removal of the head clip.
**
**  This function does NOT free the clip, it only removes it from the
** linked list.  It is the user's responsibility to manage the memory
** (the clip can be removed with a call to delete_clip();
**
**  Returns NULL on error (cliplist unitialized or attempt to remove
** using a NULL cursor;
**
**
** INPUTS:
**  struct prj_clip_list    *cliplist  -- cliplist to remove from;
**
** OUTPUTS:
**
** RETURNS:
**  struct prj_clip_struct  *  -- clip entry removed from list
**                            NULL if error;
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
struct prj_clip_struct *remove_clip( struct prj_clip_list    *cliplist)
{

    struct prj_clip_struct  *temp_clip;

    /* sanity checks */
    if ( (cliplist->status != INITIALIZED) || (cliplist->cursor == NULL) )
    {
        return (NULL);
    }

    temp_clip = cliplist->cursor;

    if (temp_clip->prev != NULL)   // check for "head"
    {
        temp_clip->next->prev = temp_clip->prev;
        cliplist->cursor = temp_clip->prev;
    }
    else
    {
        temp_clip->next->prev = NULL;
        cliplist->head = temp_clip->next;
        cliplist->cursor = temp_clip->next;
    }

    if (temp_clip->next != NULL)  // check for "tail"
    {
        temp_clip->next->prev = temp_clip->prev;
    }
    else
    {
        if (temp_clip->prev == NULL)
        {
            // we just removed the last clip
            cliplist->head = NULL;
            cliplist->tail = NULL;
            cliplist->cursor = NULL;
        }
        else
        {
            temp_clip->prev->next = NULL;
            cliplist->tail = temp_clip->prev;
            cliplist->cursor = temp_clip->prev;
        }
    }

    cliplist->num_elements--;

    return(temp_clip);
}


/*-- allocate clip --*/

/****************************************************************************
**
** struct prj_clip_struct *alloc_clip(void)
**
** DESC:
**  allocates a project clip structure, returns the pointer.
**
**  this function may be expanded to actually fill in the structure
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
struct prj_clip_struct *alloc_clip(void)
{
    struct prj_clip_struct  *temp_clip;

    if( (temp_clip = calloc( 1, sizeof(struct prj_clip_struct))) == NULL)
    {
        return (NULL);
    }

    /* make sure the internal pointers are nulled out */
    temp_clip->volume = NULL;
    temp_clip->path = NULL;
    temp_clip->name = NULL;


    return(temp_clip);
}






/*-- delete clip --*/


/****************************************************************************
**
** void free_clip( struct prj_clip_struct *clip)
**
** DESC:
**  frees a project clip structure, and the memory allocated for the
** strings and other dynamic memory items.
**
**  If the pointers should not be freed (allocated through some other
** means than calloc or malloc), they should be nulled out before
** the funciton is called.
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
void free_clip( struct prj_clip_struct *clip)
{

    /* free the internal pointers */

    if (clip->volume != NULL)
    {
        free(clip->volume);
    }

    if (clip->path != NULL)
    {
        free(clip->path);
    }

    if (clip->name != NULL)
    {
        free (clip->path);
    }


    /* free the clip pointer itself */
    free (clip);


    return();
}



/*-- move cursor to clip # --*/

/****************************************************************************
**
** int  move_clip_cursor_num( struct prj_clip_list *cliplist,
**                            int move_type, int num)
**
** DESC:
**   This function moves the current cliplist cursor the specified number of
** clips (either an absolute number or relative to the current clip cursor).
**
** CLIP_ABSOLUTE -- move the specified clip number rel to head (head = 0);
** CLIP_RELATIVE -- move relative to the current cursor;
**
**
**
** INPUTS:
**  struct prj_clip_list  *cliplist;   -- cliplist to work in
**  int    move_type;                  -- CLIP_ABSOUTE, CLIP_RELATIVE;
**  int    num;
**
** OUTPUTS:
**
** RETURNS:
**  OK  -- success;
**  ERROR -- failure (can't reach clip number or list not initialized)
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  move_clip_cursor_num( struct prj_clip_list *cliplist,
                           int move_type, int num)
{

    int i;

    if (cliplist->status != INITIALIZED)
    {
        return(ERROR);
    }

    /* move to the cursor */

    if (move_type == CLIP_ABSOLUTE)
    {
        if (num < 0)
        {
            return (ERROR);
        }

        cliplist->cursor = head;

        for (i=0; i<num; i++)
        {
            if (cliplist->cursor->next == NULL)  //tail
            {
                return (ERROR);
            }
            cliplist->cursor = cliplist->cursor->next;
        }
    }

    else if (move_type == CLIP_RELATIVE)
    {

        if (num > 0)
        {

            for (i=0; i<num; i++)
            {
                if (cliplist->cursor->next == NULL)
                {
                    return(ERROR);
                }
                cliplist->cursor = cliplist->cursor->next;
            }
        }
        else if (num < 0)
        {
            for( i=0; i>num; i--)
            {
                if (cliplist->cursor->prev == NULL)
                {
                    return(ERROR);
                }
                cliplist->cursor = cliplist->cursor->prev;
            }
        }
    }

    return(OK);
}



/*-- update the cliplist's display info --*/


/****************************************************************************
**
** int update_cliplist_display_info( struct prj_clip_list *cliplist)
**
** DESC:
**
**  update_cliplist_display_info() updates the display-related info in
** a cliplist.  Using the global variables edit_start_frame and edit_end
** frame, this function locates all clips within that range and updates
** their drawing attributes.  It does not change the structures of any
** of the other clips, so care must be used to ensure that only those
** clips from the last call to the function are re-drawn.
**
** the cursor is not affected;
**
**
**
** INPUTS:
**  struct prj_clip_list  *cliplist;
**
** OUTPUTS:
**
** RETURNS:
**  OK
**  ERROR
**
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**  !! modify later to search forward and backward based on currently
** displayed clips,  keep from searching whole list.
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int update_cliplist_display_info( struct prj_clip_list *cliplist)
{

    struct prj_clip_struct  *tempcur;
    int                     quitflag = FALSE;
    int                     visflag;

    if (cliplist->status != INITIALIZED)
    {
        return(ERROR);
    }

    /* find the first clip in the range */

    tempcur = cliplist->head;
    cliplist->displ_start = NULL;
    cliplist->num_visible = 0;

    /* scan forward to find displ_start */

    while ( (tempcur != NULL) && (!quitflag) )
    {
        visflag = FALSE;

        /* is it within the window? */

        if( (tempcur->prj_start_frame <= edit_start_frame) &&
            (tempcur->prj_end_frame >= edit_start_frame) )
        {
            visflag = TRUE;
        }
        else if (tempcur->prj_start_frame < edit_end_frame)
        {
            visflag = TRUE;
        }

        /* if so, is the first one? */

        if( visflag == TRUE )
        {
            if (cliplist->displ_start == NULL)
            {
                cliplist->displ_start = tempcur;
            }

            cliplist->displ_end   = tempcur;   /* track last */
        }
        else
        {
            if (cliplist->displ_start != NULL)
            {
                /* no more clips in view, no need to keep going! */
                quitflag = TRUE;
            }
        }

        /* next clip */
        tempcur = tempcur->next;
    }

    return(OK);


}



/*-- draw clip  --*/


/****************************************************************************
**
** int  draw_clip(struct prj_clip_list *cliplist,
**                struct prj_clip_struct *clip, int mode)
**
** DESC:
**  draw_clip draws the current clip in the specified mode (normal or
** selected).  The drawing-related items in the clip are updated.
**
**  If the mode is selected, the bar is rendered in the highlight color
** and ghost line is drawn showing the full size of the clip (not just
** the parts actually being played.  To unselect, the function should be
** called with the MODE_UNSELECT flag;
**
**
**
**
**
**
**
** INPUTS:
**  struct prj_clip_list    *cliplist;  -- needed for some drawing items
**  struct prj_clip_struct  *clip;      -- clip pointer
**  int                     mode;       -- MODE_NORMAL, MODE_SELECT
**                                         MODE_UNSELECT
**
** OUTPUTS:
**
** RETURNS:
**  OK
**  ERROR
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  draw_clip(struct prj_clip_list *cliplist,
               struct prj_clip_struct *clip, int mode)
{

    int                     i, temp_y;
    unsigned long int       temp_frame;
    unsigned short int      extents_height, clipwidth;
    char                    *

    /*! error checking */



    /* transform and clip the points */

    extents_height = (cliplib->extents.bottom - cliplist->extents.top);

    for( i=0; i< clip->num_env_pts; i++)
    {

        temp_frame = clip->aud_env[i].frame;

        if (temp_frame < edit_start_frame)
        {
            temp_frame = edit_start_frame;
        }
        else if (temp_frame > edit_end_frame)
        {
            temp_frame = edit_end_frame;
        }

        /* convert to screen coords */
        clip->aud_env[i].x_coord =(unsigned short int)(cliplist->extents.right+
                                  (int)(edit_pix_frame * (double)temp_frame));

        /* calculate the y coord */
        clip->aud_env[i].y_coord = (unsigned short int) (
                        cliplib->extents.bottom -
                        (unsigned short int)
                        ( clip->aud_env[i].level * (double)(extents_height) ) );

    }

    /* the screen width of the clip will be the distance between the */
    /* 1st and last points */

    clipwidth = (clip->aud_env[(clip->num_env_pts - 1)].x_coord -
                 clip->aud_env[0].x_coord;

    /* center and clip the filename */

    strcpy(clip->clip_name, clip->name);

          


    /* build the polygon */


    /* draw the polygon, color based on  */


    /* if MODE_SELECT, clip, build and draw the filesize box */


    /* if MODE_UNSELECT, erase the filesize box */




    return(OK);
}

/*-- draw cliplist --*/


/*-- delete clip list --*/


/*-- sort cliplist --*/


/*-- move cursor to clip closest to frame # --*/





/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

