/****************************************************************************
**
**                            Utility Library
**
**  This file contains collection of general utility functions that
** don't really belong in any other function library.
**
**
** Changelog:
**
**
** 23 Mar 97:  Added center_text function
**
** Pre 23 Mar 97:  didn't bother with log since I was only user
**
**
**
****************************************************************************/


#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <exec/types.h>
#include <exec/libraries.h>
#include <dos/dos.h>
#include <intuition/intuition.h>
#include <intuition/intuitionbase.h>


#include <graphics/text.h>
#include <graphics/rastport.h>
#include <graphics/displayinfo.h>
#include <libraries/dos.h>
#include <libraries/asl.h>

#include <clib/exec_protos.h>
#include <clib/intuition_protos.h>
#include <clib/graphics_protos.h>
#include <clib/asl_protos.h>
#include <clib/dos_protos.h>


/*-- User Include Files --*/

#include "rfx_common.h"
#include "pw_util_lib.h"



/****************************************************************************
**
** struct Border  *create_win_border( int width,  int height, int thinkness,
**                                    int orient_flag, int lightpen,
**                                    int darkpen, int frameflag, int framepen)
**
** DESC:
**   create_win_border() allocates a linked list of 2 or 3 border structures
** to define a beveled set of border lines to create a 3D raised or lowered
** effect using the specified pens.
**
** The convention is that the rectangle is lighted from the upper left, so
** a raised border has the light color in the upper-left corner and dark
** corner in the lower-right.  A Lowered border is the reverse.
**
** If the frameflag is set, a 3rd border strucutre will be added that contains
** a one-pixel box around the entire border
**
** If the orientation is neither RAISED or LOWERED, the function will return
** with the current value of the head border structure, if there is one
** (this would be either NULL, or the frame-only Border)
**
** The borders are always assumed to start at 0,0
**
**
**
** INPUTS:
**   int  width        -- width of window
**   int  height       -- height of window
**   int  thinkness    -- number of lines of width for the border
**   int  orient_flag  -- RAISED and LOWERED
**   int  lightpen     -- pen number for the light color
**   int  darkpen      -- pen number for the dark color
**   int  frameflag    -- flag for a border (TRUE or FALSE)
**   int  framepen     -- pen number for the border color
**
** OUTPUTS: none
**
** RETURNS:
**   struct Border *   -- pointer of the head border structure (NULL if error)
**
** ERRORS:
**  - Unable to allocate memory
**
** GLOBALS: none
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

struct Border  *create_win_border( int width,  int height, int thickness,
                                   int orient_flag, int lightpen,
                                   int darkpen, int frameflag, int framepen )
{

    int               num_points;
    int               new_width, new_height;
    int               save_width, save_height;
    int               save_top, save_left;
    int               new_top, new_left;
    int               i;
    int               ul_pen, lr_pen;
    struct Border     *border_head = NULL;
    struct Border     *last_border=NULL, *new_border=NULL;
    WORD              *border_data, *word_ptr;

    /* calculate number of points required */
    num_points = thickness * 3;

    /* if required, allocate and create the outer frame */

    if(frameflag == TRUE)
    {

        if( (border_data = (WORD *)calloc(10, sizeof(WORD)) ) == NULL )
        {
            return(NULL);
        }


        /* allocate the border structure */
        if( (border_head = (struct Border *)
                           calloc(1, sizeof(struct Border)) )  == NULL )
        {
            free(border_data);
            return (NULL);
        }

        /* fill out the data points and structure */

        new_width = width - 1;     /* decremnt to get screen coord */
        new_height = height - 1;

        border_data[0] = 0;              /* x0 */
        border_data[1] = 0;              /* y0 */
        border_data[2] = new_width;
        border_data[3] = 0;
        border_data[4] = new_width;
        border_data[5] = new_height;
        border_data[6] = 0;
        border_data[7] = new_height;
        border_data[8] = 0;
        border_data[9] = 0;

        new_top = 1;
        new_left = 1;

        border_head->LeftEdge    = 0;
        border_head->TopEdge     = 0;
        border_head->FrontPen    = framepen;
        border_head->BackPen     = NULL;
        border_head->DrawMode    = JAM1;
        border_head->Count       = 5;
        border_head->XY          = border_data;
        border_head->NextBorder  = NULL;

        last_border = border_head; /* keep track of "cursor" ptr */

    }
    else
    {
        /* no frame, make sure border_head is 0 */
        border_head = NULL;
        last_border = NULL;
        new_top = 0;
        new_left = 0;
        new_width = width;
        new_height = height;
    }

    save_width = new_width;
    save_height = new_height;
    save_top = new_top;
    save_left = new_left;


    /* determine the upper-left and lower-right pen colors from orientation */
    /* if it is neither RAISED or LOWERED return from the function with the */
    /* current value of the border_head pointer */

    if (orient_flag == RAISED)
    {
        ul_pen = lightpen;
        lr_pen = darkpen;
    }
    else if (orient_flag == LOWERED)
    {
        ul_pen = darkpen;
        lr_pen = lightpen;
    }
    else
    {
        return (border_head);
    }


    /* do the upper right lines */

    /* Allocate storage for data array */
    if( (border_data = (WORD *)
                      calloc( (2 * num_points), sizeof (WORD)) ) == NULL )
    {
        free_win_border( border_head );
        return (NULL);
    }

    /* allocate the border structure */
    if( (new_border = (struct Border *)
                       calloc( 1, sizeof (struct Border)) ) == NULL )
    {
        free_win_border( border_head );
        return (NULL);
    }

    /* attach to "cursor" border struct, if "cursor" is null, make head */

    if( last_border != NULL )
    {
        /* attach */
        last_border->NextBorder = new_border;
    }
    else
    {
        border_head = new_border;
    }

    /* build the polyline (3 segs/line) and fill out the structure */
    word_ptr = border_data;

    for( i=0; i<thickness; i++ )
    {

        new_width  = new_width - 1;     /* decremnt to get screen coord */
        new_height = new_height - 1;


        /* alternate the lines in the polyline */

        if ( (i % 2) == 0 )
        {
            *word_ptr = new_left;     /* x */
            word_ptr++;
            *word_ptr = new_height;   /* y */
            word_ptr++;
            *word_ptr = new_left;
            word_ptr++;
            *word_ptr = new_top;
            word_ptr++;
            *word_ptr = new_width;
            word_ptr++;
            *word_ptr = new_top;
            word_ptr++;
        }
        else
        {
            *word_ptr = new_width;     /* x */
            word_ptr++;
            *word_ptr = new_top;   /* y */
            word_ptr++;
            *word_ptr = new_left;
            word_ptr++;
            *word_ptr = new_top;
            word_ptr++;
            *word_ptr = new_left;
            word_ptr++;
            *word_ptr = new_height;
            word_ptr++;
        }

        new_top++;
        new_left++;
    }

    new_border->LeftEdge    = 0;
    new_border->TopEdge     = 0;
    new_border->FrontPen    = ul_pen;
    new_border->BackPen     = NULL;
    new_border->DrawMode    = JAM1;
    new_border->Count       = num_points;
    new_border->XY          = border_data;
    new_border->NextBorder  = NULL;

    last_border = new_border; /* keep track of "cursor" ptr */

    /*-- end of upper-left border --*/

    /* do the upper right lines */


    new_width = save_width;
    new_height = save_height;
    new_top = save_top;
    new_left = save_left;

    /* Allocate storage for data array */
    if( (border_data = (WORD *)
                      calloc( (2 * num_points), sizeof(WORD)) ) == NULL )
    {
        free_win_border( border_head );
        return (NULL);
    }

    /* allocate the border structure */
    if( (new_border = (struct Border *)
                       calloc(1, sizeof(struct Border)) ) == NULL )
    {
        free_win_border( border_head );
        return (NULL);
    }

    /* attach to "cursor" border struct, if "cursor" is null, make head */

    if( last_border != NULL )
    {
        /* attach */
        last_border->NextBorder = new_border;
    }
    else
    {
        border_head = new_border;
    }

    /* build the polyline (3 segs/line) and fill out the structure */
    word_ptr = border_data;

    for( i=0; i<thickness; i++ )
    {

        new_width  = new_width - 1;     /* decremnt to get screen coord */
        new_height = new_height - 1;


        /* alternate the lines in the polyline */

        if ( (i % 2) == 0 )
        {
            *word_ptr = new_width;     /* x */
            word_ptr++;
            *word_ptr = new_top;   /* y */
            word_ptr++;
            *word_ptr = new_width;
            word_ptr++;
            *word_ptr = new_height;
            word_ptr++;
            *word_ptr = new_left;
            word_ptr++;
            *word_ptr = new_height;
            word_ptr++;
        }
        else
        {
            *word_ptr = new_left;     /* x */
            word_ptr++;
            *word_ptr = new_height;   /* y */
            word_ptr++;
            *word_ptr = new_width;
            word_ptr++;
            *word_ptr = new_height;
            word_ptr++;
            *word_ptr = new_width;
            word_ptr++;
            *word_ptr = new_top;
            word_ptr++;
        }

        new_top++;
        new_left++;
    }

    new_border->LeftEdge    = 0;
    new_border->TopEdge     = 0;
    new_border->FrontPen    = lr_pen;
    new_border->BackPen     = NULL;
    new_border->DrawMode    = JAM1;
    new_border->Count       = num_points;
    new_border->XY          = border_data;
    new_border->NextBorder  = NULL;

    last_border = new_border; /* keep track of "cursor" ptr */


    /* return */
    return (border_head);


}


/****************************************************************************
**
** void free_win_border( struct Border *head)
**
** DESC:
**   free_win_border() frees the border structures created by
**  create_win_border.  It does this by simply walking down the linked
**  list of borders and freeing the border strucutre and data blocks.
**
**  This function assumes the memory was allocated with the ANSI calloc
** function, if another memory functions (AllocMem) was used, or the memory
** was statically created, the function will likely crash the system.
**
**
** INPUTS:
**   struct Border *head   -- pointer to the first border in the list
**
** OUTPUTS: none
**
** RETURNS: void
**
** ERRORS:
**  incorrect border, or memory allocated by method other than calloc/malloc
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/

void free_win_border (struct Border *head)
{

    struct Border *curr_border, *temp_border;

    /* if head is NULL, exit */
    if (head == NULL)
    {
        return;
    }

    /* walk down the linked list, freeing memory */
    curr_border = head;

    while (curr_border != NULL)
    {
        free( curr_border->XY );

        temp_border = curr_border;
        curr_border = curr_border->NextBorder;

        free(temp_border);
    }

    return;
}



/****************************************************************************
**
** int shift_border( struct Border *head, WORD leftshift, WORD topshift)
**
** DESC:
**  shift_border() provides a constant shift value to the LeftEdge and
** TopEdge of each element in the border structure.
**
**
** INPUTS:
**   struct Border *head   -- pointer to the first border in the list
**   WORD          leftshift  -- shift to apply to LeftEdge
**   WORD          topshift   -- shift to apply to TopEdge
**
** OUTPUTS: none
**
** RETURNS:
**   OK
**   ERROR
**
** ERRORS:
**  NULL border
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  shift_border( struct Border *head, WORD leftshift, WORD topshift)
{

    struct Border *curr_border;

    /* if head is NULL, exit */
    if (head == NULL)
    {
        return(ERROR);
    }

    /* walk down the linked list, freeing memory */
    curr_border = head;

    while (curr_border != NULL)
    {
        curr_border->LeftEdge += leftshift;
        curr_border->TopEdge += topshift;
        curr_border = curr_border->NextBorder;
    }

    return (OK);
}





/****************************************************************************
**
** int  in_hitbox(WORD x, WORD y, struct hitbox_struct hbox)
**
** DESC:
**  in_hitbox() returns TRUE if the indicated point is in or on the
** hitbox and FALSE if it isn't
**
**
** INPUTS:
**  WORD          x, y   -- x and y coordinates to test
**  struct hitbox hbox   -- hitbox struct for testing
**
** OUTPUTS:
**
** RETURNS:
**   TRUE   -- coords are in or on the hitbox
**   FALSE  -- coords are not in or on hitbox
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int  in_hitbox(WORD x, WORD y, struct hitbox_struct hbox)
{


//printf("mx = %d  my = %d  h.l = %d  h.r = %d  h.t = %d  h.b = %d\n",
//        x, y, hbox.left, hbox.right, hbox.top, hbox.bottom);


    if ( (x >= hbox.left) && ( x <= hbox.right) && (y >= hbox.top) &&
         (y <= hbox.bottom) )
    {
        return(TRUE);
    }
    else
    {
        return(FALSE);
    }

    return (FALSE);  // can't hit this, provided to squelch compiler warning
}





/****************************************************************************
**
**   int center_text( struct RastPort *rport, int width, int height,
**                      char *text, WORD *xoffset, WORD *yoffset)
**
**
** DESC:
**
**   center_text() is a simple function to provide the offsets to center
**  a text string within a box, using the default font for the specified
**  RastPort.
**
**   The function does no clipping of the text.  If the text overruns
**  the box, it is just sets the dimension to zero.
**
**
**
** INPUTS:
**  struct RastPort *rport       -- ptr to rasterport for rendering
**  WORD            width        -- width of box for centering
**  WORD            height       -- height of box for centering
**  char            *text          -- ptr to text to be clipped
**
**
** OUTPUTS:
**  WORD          *xoffset       -- X offset to center text
**  WORD          *yoffset       -- Y offset to center text
**
** RETURNS: (int)
**
**  OK
**  ERROR
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
int center_text( struct RastPort *rport, int width, int height,
                   char *text, WORD *xoffset, WORD *yoffset)
{

    int                 text_len;
    struct TextExtent   t_ext;



    /* initialize variables / error checking */

    if ( (text == NULL) || (rport == NULL) )
    {
        return (ERROR);
    }

    if ( (width <=0) || (height <= 0) )
    {
        return (ERROR);
    }


    text_len = strlen(text);


    /* find the size of the string and center it */

    TextExtent( rport, (STRPTR)text, text_len, &t_ext);

    *xoffset=(t_ext.te_Width<width) ? (WORD)((width-t_ext.te_Width)/2):0;

    *yoffset=(t_ext.te_Height<height) ?
             (WORD)(((height-t_ext.te_Height)/2)+(t_ext.te_Height/2)):0;

    return(OK);

}






/****************************************************************************
**
**   char *clip_text( struct RastPort *rport, int max_text_width, int clipdir,
**                    char *text, int *out_strlen, int *text_width_out)
**
**
** DESC:
**   clip_text() clips a test string to fit within a specified text width
**  if rendered with the current font for the specified RasterPort.  If the
**  text is wider than
**  the allowed width, characters will be clipped from the string in the
**  direction indicated by the clipdir flag (either clipping text from the
**  front or back).  The function returns a pointer to the text string
**  which can be safely rendered in the line (this is a DESTRUCTIVE operation,
**  if the string is clipped from the end, it will replace characters with
**  NULLs to shorten the string).
**
**   The final width of the rendered text
**  string (in pixels) is also returned through the text_width_out pointer).
**  the number of characters in the string is also returned so the caller
**  will not have to call strlen again.
**
**
** INPUTS:
**  struct RastPort *rport       -- ptr to rasterport for rendering
**  int           max_text_width -- maximum width for text
**  int           clipdir        -- CLIP_FORWARD or CLIP_BACKWARD
**  char          *text          -- ptr to text to be clipped
**
**
** OUTPUTS:
**  int           *out_strlen       -- # chars in new string
**  int           *text_width_out  -- ptr to width of rendered text
**
** RETURNS:
**  char          *  -- pointer to the text block to be rendered,
**                          this is either NULL (some error has occured),
**                          the same as the source pointer (if all text
**                          is there or clipped CLIP_BACKWARD), or different
**                          from the source (clipped CLIP_FORWARD)
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/
char *clip_text( struct RastPort *rport, int max_text_width, int clipdir,
                 char *text, int *out_strlen, int *text_width_out)
{

    int                 text_len;
    char                *txt_tail_null, *txt_head;
    struct TextExtent   t_ext;


    /* initialize variables / error checking */

    if (text == NULL)
    {
        return (NULL);
    }

    text_len = strlen(text);

    txt_head      = text;
    txt_tail_null = &text[text_len];

    if ( (clipdir != CLIP_FORWARD) && (clipdir != CLIP_BACKWARD) )
    {
        return(NULL);
    }

    /* trim text size until it fits! */

    TextExtent( rport, (STRPTR)txt_head, text_len, &t_ext);

    while( (t_ext.te_Width >= max_text_width) && (text_len > 0 ) )
    {
        /* step one character forward or backward */
        if (clipdir == CLIP_FORWARD)
        {
            txt_head++;
        }
        else
        {
            txt_tail_null--;
            *txt_tail_null = '\0';
        }
        text_len--;

        TextExtent( rport, (STRPTR)txt_head, text_len, &t_ext);

    }

    /* set the output variables and return the pointer */
    *out_strlen = text_len;
    *text_width_out = t_ext.te_Width;

    return(txt_head);

}






/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/








/*--- Spare Template ---*/


/****************************************************************************
**
**
** DESC:
**
**
** INPUTS:
**
** OUTPUTS:
**
** RETURNS:
**
** ERRORS:
**
** GLOBALS:
**
**
** CALLS:
**
** NOTES:
**
**
** HISTORY:
**
**     DATE         By     Revision     Note
**
**
****************************************************************************/



